@@ -25,8 +25,6 @@ struct _CallbackInfo {
 
 static GNOME_Panel panel_client = CORBA_OBJECT_NIL;
 
-#define APPLET_ID_KEY "applet_id_key"
-#define APPLET_WIDGET_KEY "applet_widget_key"
 #define pg_return_if_fail(evp,x) {if(!(x)) { g_warning("file %s: line %d: Corba Exception: type = %d exid = %s\n", __FILE__, __LINE__, (evp)->_major, (evp)->_repo_id); return; }}
 #define pg_return_val_if_fail(evp,x,y) {if(!(x)) { g_warning("file %s: line %d: Corba Exception: type = %d exid = %s\n", __FILE__, __LINE__, (evp)->_major, (evp)->_repo_id); return y;}}
 
@@ -49,79 +47,79 @@ static PortableServer_ServantBase__epv base_epv = {
 };
 
 static void
-server_applet_change_orient(CustomAppletServant *servant,
-			    GNOME_Panel_OrientType orient,
+server_applet_change_orient(PortableServer_Servant _servant,
+			    const GNOME_Panel_OrientType orient,
 			    CORBA_Environment *ev);
 
 static void
-server_applet_change_size(CustomAppletServant *servant,
-			  CORBA_short size,
+server_applet_change_size(PortableServer_Servant _servant,
+			  const CORBA_short size,
 			  CORBA_Environment *ev);
 
 static void
-server_applet_do_callback(CustomAppletServant *servant,
-			  CORBA_char * callback_name,
+server_applet_do_callback(PortableServer_Servant _servant,
+			  const CORBA_char * callback_name,
 			  CORBA_Environment *ev);
 
 static void
-server_applet_save_session(CustomAppletServant *servant,
-			   CORBA_char * cfgpath,
-			   CORBA_char * globcfgpath,
-			   CORBA_unsigned_long cookie,
+server_applet_save_session(PortableServer_Servant _servant,
+			   const CORBA_char * cfgpath,
+			   const CORBA_char * globcfgpath,
+			   const CORBA_unsigned_long cookie,
 			   CORBA_Environment *ev);
 
 static CORBA_boolean
-server_applet_session_save(CustomAppletServant *servant,
-			   CORBA_char * cfgpath,
-			   CORBA_char * globcfgpath,
+server_applet_session_save(PortableServer_Servant _servant,
+			   const CORBA_char * cfgpath,
+			   const CORBA_char * globcfgpath,
 			   CORBA_Environment *ev);
 
 static void
-server_applet_back_change(CustomAppletServant *servant,
-			  GNOME_Panel_BackInfoType *backing,
+server_applet_back_change(PortableServer_Servant _servant,
+			  const GNOME_Panel_BackInfoType *backing,
 			  CORBA_Environment *ev);
 
 static void
-server_applet_draw(CustomAppletServant *servant,
+server_applet_draw(PortableServer_Servant _servant,
 		   CORBA_Environment *ev);
 
 static void
-server_applet_set_tooltips_state(CustomAppletServant *servant,
-				 CORBA_boolean enabled,
+server_applet_set_tooltips_state(PortableServer_Servant _servant,
+				 const CORBA_boolean enabled,
 				 CORBA_Environment *ev);
 
 static void
-server_applet_change_position(CustomAppletServant *servant,
-			      CORBA_short x,
-			      CORBA_short y,
+server_applet_change_position(PortableServer_Servant _servant,
+			      const CORBA_short x,
+			      const CORBA_short y,
 			      CORBA_Environment *ev);
 
 static CORBA_char *
-server_applet__get_goad_id(CustomAppletServant *servant,
+server_applet__get_goad_id(PortableServer_Servant _servant,
 			   CORBA_Environment *ev);
 
 static void
-server_applet_freeze_changes(CustomAppletServant *servant,
+server_applet_freeze_changes(PortableServer_Servant _servant,
 			     CORBA_Environment *ev);
 
 static void
-server_applet_thaw_changes(CustomAppletServant *servant,
+server_applet_thaw_changes(PortableServer_Servant _servant,
 			   CORBA_Environment *ev);
 
 static POA_GNOME_Applet__epv applet_epv = {
   NULL,
-  (gpointer)&server_applet_change_orient,
-  (gpointer)&server_applet_do_callback,
-  (gpointer)&server_applet_session_save,
-  (gpointer)&server_applet_back_change,
-  (gpointer)&server_applet_set_tooltips_state,
-  (gpointer)&server_applet__get_goad_id,
-  (gpointer)&server_applet_draw,
-  (gpointer)&server_applet_save_session,
-  (gpointer)&server_applet_change_size,
-  (gpointer)&server_applet_change_position,
-  (gpointer)&server_applet_freeze_changes,
-  (gpointer)&server_applet_thaw_changes
+  server_applet_change_orient,
+  server_applet_do_callback,
+  server_applet_session_save,
+  server_applet_back_change,
+  server_applet_set_tooltips_state,
+  server_applet__get_goad_id,
+  server_applet_draw,
+  server_applet_save_session,
+  server_applet_change_size,
+  server_applet_change_position,
+  server_applet_freeze_changes,
+  server_applet_thaw_changes
 };
 
 static POA_GNOME_Applet__vepv vepv = { &base_epv, &applet_epv };
@@ -149,17 +147,11 @@ typedef void (*PositionSignal) (GtkObject * object,
 				int y,
 				gpointer data);
 
-typedef void (*DeprecSizeSignal) (GtkObject * object,
-				  GNOME_Panel_SizeType type,
-				  gpointer data);
-
 static int applet_count = 0;
 
-static int die_on_last = FALSE;
-
+static gboolean die_on_last = FALSE;
 static GtkPlugClass *parent_class;
-
-static GtkTooltips *applet_tooltips=NULL;
+static GtkTooltips *applet_tooltips = NULL;
 
 #define CD(applet) ((CustomAppletServant *)APPLET_WIDGET(applet)->corbadat)
 
@@ -188,7 +180,6 @@ applet_widget_get_type ()
 enum {
 	CHANGE_ORIENT_SIGNAL,
 	CHANGE_PIXEL_SIZE_SIGNAL,
-	CHANGE_SIZE_SIGNAL,
 	SAVE_SESSION_SIGNAL,
 	BACK_CHANGE_SIGNAL,
 	DO_DRAW_SIGNAL,
@@ -253,24 +244,6 @@ marshal_signal_position (GtkObject * object,
 		  func_data);
 }
 
-static void
-marshal_signal_deprec_size (GtkObject * object,
-			    GtkSignalFunc func,
-			    gpointer func_data,
-			    GtkArg * args)
-{
-	DeprecSizeSignal rfunc;
-
-	g_warning("change_size signal is deprecated, use the new "
-		  "change_pixel_size signal");
-
-	rfunc = (DeprecSizeSignal) func;
-
-	(*rfunc) (object, GTK_VALUE_ENUM (args[0]),
-		  func_data);
-}
-
-
 static void
 applet_widget_class_init (AppletWidgetClass *class)
 {
@@ -300,16 +273,6 @@ applet_widget_class_init (AppletWidgetClass *class)
 			       GTK_TYPE_NONE,
 			       1,
 			       GTK_TYPE_INT);
-	applet_widget_signals[CHANGE_SIZE_SIGNAL] =
-		gtk_signal_new("change_size",
-			       GTK_RUN_FIRST,
-			       object_class->type,
-			       GTK_SIGNAL_OFFSET(AppletWidgetClass,
-			       			 change_size),
-			       marshal_signal_deprec_size,
-			       GTK_TYPE_NONE,
-			       1,
-			       GTK_TYPE_ENUM);
 	applet_widget_signals[SAVE_SESSION_SIGNAL] =
 		gtk_signal_new("save_session",
 			       GTK_RUN_LAST,
@@ -368,7 +331,6 @@ applet_widget_class_init (AppletWidgetClass *class)
 				     LAST_SIGNAL);
 
 	class->change_orient = NULL;
-	class->change_size = NULL;
 	class->save_session = NULL;
 	class->back_change = NULL;
 	class->do_draw = NULL;
@@ -422,25 +384,50 @@ static void
 applet_widget_destroy(GtkWidget *w, gpointer data)
 {
 	AppletWidget *applet;
+	GtkPlug *plug;
 	CORBA_Environment ev;
 
 	g_return_if_fail(w != NULL);
 	g_return_if_fail(IS_APPLET_WIDGET(w));
 
 	applet = APPLET_WIDGET(w);
-	if(!applet->privcfgpath)
-		return;
+	plug = GTK_PLUG(w);
 
-	g_free(applet->privcfgpath);
-	g_free(applet->globcfgpath);
+	/* XXX: hackaround to broken gtkplug/gtksocket, we kill the references
+	   to ourselves on the socket and our references to the socket and
+	   destroy the socket */
+	if(plug->same_app && plug->socket_window) {
+		GtkSocket *socket;
+		gdk_window_get_user_data (plug->socket_window,
+					  (gpointer *)&socket);
+		if(socket) {
+			GtkWidget *toplevel = gtk_widget_get_toplevel (GTK_WIDGET(socket));
+			if (toplevel && GTK_IS_WINDOW (toplevel))
+				gtk_window_remove_embedded_xid (GTK_WINDOW (toplevel), 
+								GDK_WINDOW_XWINDOW (socket->plug_window));
+			socket->plug_window = NULL;
+			socket->same_app = FALSE;
+			plug->socket_window = NULL;
+			plug->same_app = FALSE;
+			gtk_widget_destroy(GTK_WIDGET(socket));
+		}
+	}
 
-	applet->privcfgpath = NULL;
-	applet->globcfgpath = NULL;
+	if(applet->privcfgpath) {
+		g_free(applet->privcfgpath);
+		g_free(applet->globcfgpath);
 
-	CORBA_exception_init(&ev);
-	if(GTK_BIN(w)->child == NULL)
-		GNOME_PanelSpot_abort_load(CD(applet)->pspot, &ev);
-	CORBA_exception_free(&ev);
+		applet->privcfgpath = NULL;
+		applet->globcfgpath = NULL;
+
+		CORBA_exception_init(&ev);
+		/* if nothing has been added as our child, this means we have
+		   not yet fully completed load, so notify the panel that we
+		   are going to die */
+		if(GTK_BIN(w)->child == NULL)
+			GNOME_PanelSpot_abort_load(CD(applet)->pspot, &ev);
+		CORBA_exception_free(&ev);
+	}
 
 	applet_servant_destroy(applet->corbadat);
 
@@ -594,7 +581,7 @@ void
 applet_widget_unregister_callback(AppletWidget *applet,
 				  const char *name)
 {
-	GSList *ltmp;
+	GSList *li;
 	CallbackInfo *cbi = NULL;
 	CORBA_Environment ev;
 
@@ -606,11 +593,11 @@ applet_widget_unregister_callback(AppletWidget *applet,
 
 	g_return_if_fail(name!=NULL);
 
-	for(ltmp = CD(applet)->callbacks;
-	    ltmp && !cbi;
-	    ltmp = g_slist_next(ltmp)) {
-		if(!strcmp(((CallbackInfo*)ltmp->data)->name, name))
-			cbi = ltmp->data;
+	for(li = CD(applet)->callbacks; li; li = g_slist_next(li)) {
+		if(!strcmp(((CallbackInfo*)li->data)->name, name)) {
+			cbi = li->data;
+			break;
+		}
 	}
 
 	if(!cbi) return;
@@ -740,11 +727,11 @@ gnome_panel_applet_corba_init(AppletWidget *applet, const char *goad_id)
 	applet_servant->servant.vepv = &vepv;
 
 	POA_GNOME_Applet__init((POA_GNOME_Applet *)applet_servant, &ev);
-
 	pg_return_val_if_fail(&ev, ev._major == CORBA_NO_EXCEPTION, NULL);
 
 	applet_servant->poa = poa = (PortableServer_POA)
 		CORBA_ORB_resolve_initial_references(orb, "RootPOA", &ev);
+	pg_return_val_if_fail(&ev, ev._major == CORBA_NO_EXCEPTION, NULL);
 
 	PortableServer_POAManager_activate(PortableServer_POA__get_the_POAManager(poa, &ev), &ev);
 	pg_return_val_if_fail(&ev, ev._major == CORBA_NO_EXCEPTION, NULL);
@@ -820,6 +807,8 @@ gnome_panel_applet_corba_init(AppletWidget *applet, const char *goad_id)
 	if(ev._major) {
 		g_warning("CORBA Exception, can't get size");
 		applet->size = PIXEL_SIZE_STANDARD;
+		/* no need to recycle the exception here, as we will free it
+		   next */
 	}
 
 	CORBA_exception_free(&ev);
@@ -828,6 +817,19 @@ gnome_panel_applet_corba_init(AppletWidget *applet, const char *goad_id)
 }
 
 
+/**
+ * applet_widget_new:
+ * @goad_id: The goad_id of the applet we are starting
+ *
+ * Description: Make a new applet and register us with the panel, if you
+ * decide to cancel the load before calling #applet_widget_add, you should
+ * call #applet_widget_abort_load.  This widget is a simple container but you
+ * should always use only #applet_widget_add to add a child and you should
+ * only use it once.
+ *
+ * Returns: A pointer to a new widget of type #AppletWidget, or %NULL if 
+ * something went wrong.
+ **/
 GtkWidget *
 applet_widget_new(const char *goad_id)
 {
@@ -839,6 +841,12 @@ applet_widget_new(const char *goad_id)
 	return GTK_WIDGET(applet);
 }
 
+/**
+ * applet_widget_construct:
+ *
+ * Description: For bindings and subclassing only
+ *
+ **/
 void
 applet_widget_construct(AppletWidget* applet, const char *goad_id)
 {
@@ -862,13 +870,22 @@ applet_widget_construct(AppletWidget* applet, const char *goad_id)
 	applet_count++;
 }
 
+/**
+ * applet_widget_get_applet_count:
+ *
+ * Description:  Gets the number of applets loaded in this this process.  If
+ * this is a shared lib applet it will return the total number of shared lib
+ * applets loaded.
+ *
+ * Returns:  The number of applets loaded.
+ **/
 int
-applet_widget_get_applet_count()
+applet_widget_get_applet_count(void)
 {
 	return applet_count;
 }
 
-static int
+static gboolean
 applet_event(GtkWidget *w,GdkEvent *event,GtkPlug *aw)
 {
 	GdkEventButton *bevent = (GdkEventButton *)event;
@@ -918,9 +935,27 @@ bind_applet_events(GtkWidget *widget, gpointer data)
 				       bind_applet_events, data);
 }
 
+/**
+ * applet_widget_add_full:
+ * @applet: the #AppletWidget to work with
+ * @widget: the child to add
+ * @bind_events: bind 2nd and 3rd button events over the applet if %TRUE
+ *
+ * Description:  Add a child (@widget) to the @applet.  This finishes the
+ * handshaking with the panel started in @applet_widget_new.  You should never
+ * call this function twice for the same @applet and you should always use
+ * this function rather then #gtk_container_add.  If you have already created
+ * an applet widget with #applet_widget_new, but need to cancel the loading
+ * of the applet, use #applet_widget_abort_load.  This function is only for
+ * special applets and you should use #applet_widget_bind_events on some
+ * internal widget if @bind_events was %FALSE.  Normally you'll just want to
+ * use #applet_widget_add.
+ *
+ * Returns:
+ **/
 void
 applet_widget_add_full(AppletWidget *applet, GtkWidget *widget,
-		       int bind_events)
+		       gboolean bind_events)
 {
 	CORBA_Environment ev;
 
@@ -940,18 +975,54 @@ applet_widget_add_full(AppletWidget *applet, GtkWidget *widget,
 		bind_applet_events(GTK_WIDGET(applet), applet);
 }
 
+/**
+ * applet_widget_add:
+ * @applet: the #AppletWidget to work with
+ * @widget: the child to add
+ *
+ * Description:  Add a child (@widget) to the @applet.  This finishes the
+ * handshaking with the panel started in @applet_widget_new.  You should never
+ * call this function twice for the same @applet and you should always use
+ * this function rather then #gtk_container_add.  If you have already created
+ * an applet widget with #applet_widget_new, but need to cancel the loading
+ * of the applet, use #applet_widget_abort_load.
+ *
+ * Returns:
+ **/
 void
 applet_widget_add(AppletWidget *applet, GtkWidget *widget)
 {
 	applet_widget_add_full(applet,widget,TRUE);
 }
 
+/**
+ * applet_widget_bind_events:
+ * @applet: the #AppletWidget to work with
+ * @widget: the widget over which to bind events
+ *
+ * Description:  Binds the 2nd and 3rd button clicks over this widget. 
+ * Normally this is done during #applet_widget_add, but if you need to
+ * bind events over a widget which you added later, use this function.
+ *
+ * Returns:
+ **/
 void
 applet_widget_bind_events(AppletWidget *applet, GtkWidget *widget)
 {
 	bind_applet_events(GTK_WIDGET(applet),widget);
 }
 
+/**
+ * applet_widget_set_widget_tooltip:
+ * @applet: the #AppletWidget to work with
+ * @widget: the widget to set tooltip on
+ * @text: the tooltip text
+ *
+ * Description:  Set a tooltip on the @widget that will follow the tooltip
+ * setting from the panel configuration.
+ *
+ * Returns:
+ **/
 void
 applet_widget_set_widget_tooltip(AppletWidget *applet,
 				 GtkWidget *widget,
@@ -965,9 +1036,19 @@ applet_widget_set_widget_tooltip(AppletWidget *applet,
 	if(!applet_tooltips)
 		applet_tooltips = gtk_tooltips_new();
 
-	gtk_tooltips_set_tip (applet_tooltips,widget,text,NULL);
+	gtk_tooltips_set_tip (applet_tooltips,  widget, text, NULL);
 }
 
+/**
+ * applet_widget_set_tooltip:
+ * @applet: the #AppletWidget to work with
+ * @text: the tooltip text
+ *
+ * Description:  Set a tooltip on the entire applet that will follow the
+ * tooltip setting from the panel configuration.
+ *
+ * Returns:
+ **/
 void
 applet_widget_set_tooltip(AppletWidget *applet, const char *text)
 {
@@ -980,8 +1061,16 @@ applet_widget_set_tooltip(AppletWidget *applet, const char *text)
 	CORBA_exception_free(&ev);
 }
 
-/* Get the orientation the applet should use */
-GNOME_Panel_OrientType
+/**
+ * applet_widget_get_panel_orient:
+ * @applet: the #AppletWidget to work with
+ *
+ * Description:  Gets the orientation of the panel this widget is on.
+ * it can be one of ORIENT_UP, ORIENT_DOWN, ORIENT_LEFT and ORIENT_RIGHT.
+ *
+ * Returns:  PanelOrientType enum of the orientation
+ **/
+PanelOrientType
 applet_widget_get_panel_orient(AppletWidget *applet)
 {
 	g_return_val_if_fail(applet != NULL,ORIENT_UP);
@@ -990,29 +1079,16 @@ applet_widget_get_panel_orient(AppletWidget *applet)
 	return applet->orient;
 }
 
-/* Get the size the applet should use */
-GNOME_Panel_SizeType
-applet_widget_get_panel_size(AppletWidget *applet)
-{
-	g_return_val_if_fail(applet != NULL,GNOME_Panel_SIZE_STANDARD);
-	g_return_val_if_fail(IS_APPLET_WIDGET(applet), GNOME_Panel_SIZE_STANDARD);
-	
-	g_warning("applet_widget_get_panel_size is deprecated, use the new pixel_size methods");
-
-	switch(applet->size) {
-	case 24:
-		return GNOME_Panel_SIZE_TINY;
-	default:
-	case 48:
-		return GNOME_Panel_SIZE_STANDARD;
-	case 64:
-		return GNOME_Panel_SIZE_LARGE;
-	case 80:
-		return GNOME_Panel_SIZE_HUGE;
-	}
-}
-
-/* Get the size the applet should use */
+/**
+ * applet_widget_get_panel_pixel_size:
+ * @applet: the #AppletWidget to work with
+ *
+ * Description:  Gets the width of the panel in pixels.  This is not the
+ * actual size, but the recomended one.  The panel may be streched if the
+ * applets use larger sizes then this.
+ *
+ * Returns:  Size of panel in pixels
+ **/
 int
 applet_widget_get_panel_pixel_size(AppletWidget *applet)
 {
@@ -1022,8 +1098,17 @@ applet_widget_get_panel_pixel_size(AppletWidget *applet)
 	return applet->size;
 }
 
-/* Get the free space for the applet if it's on an edge panel or 0
-   if on a packed panel or on error */
+/**
+ * applet_widget_get_free_space:
+ * @applet: the #AppletWidget to work with
+ *
+ * Description:  Gets the free space left that you can use for your applet.
+ * This is the number of pixels around your applet to both sides.  If you
+ * strech by this amount you will not disturb any other applets.  If you
+ * are on a packed panel 0 will be returned.
+ *
+ * Returns:  Free space left for your applet.
+ **/
 int
 applet_widget_get_free_space(AppletWidget *applet)
 {
@@ -1038,9 +1123,21 @@ applet_widget_get_free_space(AppletWidget *applet)
 	return r;
 }
 
-/* sets if the change_position signal is sent*/
+/**
+ * applet_widget_send_position:
+ * @applet: the #AppletWidget to work with
+ * @enable: whether to enable or disable change_position signal
+ *
+ * Description:  If you need to get a signal everytime this applet changes
+ * position relative to the screen, you need to run this function with %TRUE
+ * for @enable and bind the change_position signal on the applet.  This signal
+ * can be quite CPU/bandwidth consuming so only applets which need it should
+ * use it.  By default change_position is not sent.
+ *
+ * Returns:
+ **/
 void
-applet_widget_send_position(AppletWidget *applet, int enable)
+applet_widget_send_position(AppletWidget *applet, gboolean enable)
 {
 	CORBA_Environment ev;
 	g_return_if_fail(applet != NULL);
@@ -1051,9 +1148,20 @@ applet_widget_send_position(AppletWidget *applet, int enable)
 	CORBA_exception_free(&ev);
 }
 
-/* sets if the do_draw signal is sent*/
+/**
+ * applet_widget_send_draw:
+ * @applet: the #AppletWidget to work with
+ * @enable: whether to enable or disable do_draw signal
+ *
+ * Description:  If you are using rgb background drawing, call this function
+ * with %TRUE for @enable, and then bind the do_draw signal.  Inside that
+ * signal you can get an RGB buffer to draw on with #applet_widget_get_rgb_bg.
+ * The do_draw signal will only be sent when the RGB truly changed.
+ *
+ * Returns:
+ **/
 void
-applet_widget_send_draw(AppletWidget *applet, int enable)
+applet_widget_send_draw(AppletWidget *applet, gboolean enable)
 {
 	CORBA_Environment ev;
 	g_return_if_fail(applet != NULL);
@@ -1064,7 +1172,20 @@ applet_widget_send_draw(AppletWidget *applet, int enable)
 	CORBA_exception_free(&ev);
 }
 
-/* gets the rgb background, useful in conjunction with the draw signal */
+/**
+ * applet_widget_get_rgb_bg:
+ * @applet: the #AppletWidget to work with
+ * @rgb: pointer to a pointer to which the rgb buffer will be returned
+ * @w: pointer to an integer in which the width will be stored
+ * @h: pointer to an integer in which the height will be stored
+ * @rowstride: pointer to an integer in which the rowstride will be stored
+ *
+ * Description:  Gets an rgb buffer that you can draw your applet on.  Usefull
+ * in conjunction with the do_draw signal and the #applet_widget_send_draw
+ * method.  The rgb should be freed after use with g_free.
+ *
+ * Returns:
+ **/
 void
 applet_widget_get_rgb_bg(AppletWidget *applet, guchar **rgb,
 			 int *w, int *h, int *rowstride)
@@ -1129,7 +1250,21 @@ applet_widget_get_rgb_bg(AppletWidget *applet, guchar **rgb,
 	CORBA_free(image);
 }
 
-int	
+/**
+ * applet_widget_init:
+ * @app_id: applet id
+ * @app_version: applet version
+ * @argc: the argc passed to main
+ * @argv: the argv passed to main
+ * @options: extra popt options to use
+ * @flags: extra popt flags
+ * @return_ctx: return popt context
+ *
+ * Description: Initialize the applet library, gnome and corba.
+ *
+ * Returns: A boolean, %TRUE if we succeed, %FALSE if an error occured
+ **/
+gboolean
 applet_widget_init(const char *app_id,
 		   const char *app_version,
 		   int argc,
@@ -1138,8 +1273,6 @@ applet_widget_init(const char *app_id,
 		   unsigned int flags,
 		   poptContext *return_ctx)
 {
-	int ret = TRUE; /*bogus value, this should be if we succeded or not
-			  or something*/
 	CORBA_Environment ev;
 	CORBA_ORB orb;
 
@@ -1152,28 +1285,55 @@ applet_widget_init(const char *app_id,
 					       &argc, argv,
 					       options, flags, return_ctx,
 					       GNORBA_INIT_SERVER_FUNC, &ev);
+	if(ev._major != CORBA_NO_EXCEPTION) {
+		CORBA_exception_free(&ev);
+		return FALSE;
+	}
 
 	CORBA_exception_free(&ev);
 
-	return ret;
+	return TRUE;
 }
 
 /*****************************************************************************
   CORBA STUFF
  *****************************************************************************/
 
+/**
+ * applet_widget_gtk_main:
+ *
+ * Description: Run the main loop, just like #gtk_main
+ *
+ * Returns:
+ **/
 void
 applet_widget_gtk_main(void)
 {
 	gtk_main();
 }
 
+/**
+ * applet_widget_gtk_main_quit:
+ *
+ * Description: Quit the main loop, just like #gtk_main_quit
+ *
+ * Returns:
+ **/
 void
 applet_widget_gtk_main_quit (void)
 {
 	gtk_main_quit();
 }
 
+/**
+ * applet_widget_panel_quit:
+ *
+ * Description: Trigger 'Log out' on the panel.  This shouldn't be
+ * used in normal applets, as it is not normal for applets to trigger
+ * a logout.
+ *
+ * Returns:
+ **/
 void
 applet_widget_panel_quit (void)
 {
@@ -1184,11 +1344,46 @@ applet_widget_panel_quit (void)
 	CORBA_exception_free(&ev);
 }
 
+/**
+ * applet_widget_queue_resize:
+ * @applet: #AppletWidget to work on
+ *
+ * Description:  For shared library applets this calls
+ * #gtk_widget_queue_resize on the socket, for external applets this
+ * just calls this on the #AppletWidget itself, but in both cases
+ * it forces a resize of the socket on the panel
+ *
+ * Returns:
+ **/
+void
+applet_widget_queue_resize(AppletWidget *applet)
+{
+	GtkPlug *plug;
+
+	g_return_if_fail(applet != NULL);
+	g_return_if_fail(IS_APPLET_WIDGET(applet));
+
+	plug = GTK_PLUG(applet);
+
+	/* XXX: hackaround to broken gtkplug/gtksocket, we must manually
+	   queue resize on the socket as the plug does not have a parent */
+	if(plug->same_app && plug->socket_window) {
+		GtkWidget *socket;
+		gdk_window_get_user_data (plug->socket_window,
+					  (gpointer *)&socket);
+		if(socket)
+			gtk_widget_queue_resize(socket);
+	} else {
+		gtk_widget_queue_resize(GTK_WIDGET(applet));
+	}
+}
+
 static void
-server_applet_change_orient(CustomAppletServant *servant,
-			    GNOME_Panel_OrientType orient,
+server_applet_change_orient(PortableServer_Servant _servant,
+			    const GNOME_Panel_OrientType orient,
 			    CORBA_Environment *ev)
 {
+	CustomAppletServant *servant = (CustomAppletServant *)_servant;
 	servant->appwidget->orient = orient;
 	if(servant->appwidget->frozen_level>0) {
 		servant->appwidget->frozen_got_orient = TRUE;
@@ -1201,38 +1396,16 @@ server_applet_change_orient(CustomAppletServant *servant,
 }
 
 static void
-server_applet_change_size(CustomAppletServant *servant,
-			  CORBA_short size,
+server_applet_change_size(PortableServer_Servant _servant,
+			  const CORBA_short size,
 			  CORBA_Environment *ev)
 {
+	CustomAppletServant *servant = (CustomAppletServant *)_servant;
 	servant->appwidget->size = size;
 	if(servant->appwidget->frozen_level>0) {
 		servant->appwidget->frozen_got_size = TRUE;
 		servant->appwidget->frozen_size = size;
 	} else {
-		switch(size) {
-		case 24:
-			gtk_signal_emit(GTK_OBJECT(servant->appwidget),
-					applet_widget_signals[CHANGE_SIZE_SIGNAL],
-					GNOME_Panel_SIZE_TINY);
-			break;
-		default:
-		case 48:
-			gtk_signal_emit(GTK_OBJECT(servant->appwidget),
-					applet_widget_signals[CHANGE_SIZE_SIGNAL],
-					GNOME_Panel_SIZE_STANDARD);
-			break;
-		case 64:
-			gtk_signal_emit(GTK_OBJECT(servant->appwidget),
-					applet_widget_signals[CHANGE_SIZE_SIGNAL],
-					GNOME_Panel_SIZE_LARGE);
-			break;
-		case 80:
-			gtk_signal_emit(GTK_OBJECT(servant->appwidget),
-					applet_widget_signals[CHANGE_SIZE_SIGNAL],
-					GNOME_Panel_SIZE_HUGE);
-			break;
-		}
 		gtk_signal_emit(GTK_OBJECT(servant->appwidget),
 				applet_widget_signals[CHANGE_PIXEL_SIZE_SIGNAL],
 				size);
@@ -1240,10 +1413,11 @@ server_applet_change_size(CustomAppletServant *servant,
 }
 
 static void
-server_applet_do_callback(CustomAppletServant *servant,
-			  CORBA_char * callback_name,
+server_applet_do_callback(PortableServer_Servant _servant,
+			  const CORBA_char * callback_name,
 			  CORBA_Environment *ev)
 {
+	CustomAppletServant *servant = (CustomAppletServant *)_servant;
 	GSList *list;
 	CallbackInfo *info;
 
@@ -1260,12 +1434,13 @@ server_applet_do_callback(CustomAppletServant *servant,
 
 /* this is the new session saving call and the one which should be used */
 static void
-server_applet_save_session(CustomAppletServant *servant,
-			   CORBA_char * cfgpath,
-			   CORBA_char * globcfgpath,
-			   CORBA_unsigned_long cookie,
+server_applet_save_session(PortableServer_Servant _servant,
+			   const CORBA_char * cfgpath,
+			   const CORBA_char * globcfgpath,
+			   const CORBA_unsigned_long cookie,
 			   CORBA_Environment *ev)
 {
+	CustomAppletServant *servant = (CustomAppletServant *)_servant;
 	AppletWidget *applet;
 	char *cfg = g_strdup(cfgpath);
 	char *globcfg = g_strdup(globcfgpath);
@@ -1289,11 +1464,12 @@ server_applet_save_session(CustomAppletServant *servant,
 /* this is here just that if an applet uses the new lib with the old
    panel it will still work */
 static CORBA_boolean
-server_applet_session_save(CustomAppletServant *servant,
-			   CORBA_char * cfgpath,
-			   CORBA_char * globcfgpath,
+server_applet_session_save(PortableServer_Servant _servant,
+			   const CORBA_char * cfgpath,
+			   const CORBA_char * globcfgpath,
 			   CORBA_Environment *ev)
 {
+	CustomAppletServant *servant = (CustomAppletServant *)_servant;
 	AppletWidget *applet;
 	char *cfg = g_strdup(cfgpath);
 	char *globcfg = g_strdup(globcfgpath);
@@ -1311,10 +1487,11 @@ server_applet_session_save(CustomAppletServant *servant,
 }
 
 static void
-server_applet_back_change(CustomAppletServant *servant,
-			  GNOME_Panel_BackInfoType *backing,
+server_applet_back_change(PortableServer_Servant _servant,
+			  const GNOME_Panel_BackInfoType *backing,
 			  CORBA_Environment *ev)
 {
+	CustomAppletServant *servant = (CustomAppletServant *)_servant;
 	GdkColor color={0,0,0,0}, *cptr = NULL;
 	char *pptr = NULL;
 
@@ -1351,18 +1528,20 @@ server_applet_back_change(CustomAppletServant *servant,
 }
 
 static void
-server_applet_draw(CustomAppletServant *servant,
+server_applet_draw(PortableServer_Servant _servant,
 		   CORBA_Environment *ev)
 {
+	CustomAppletServant *servant = (CustomAppletServant *)_servant;
 	gtk_signal_emit(GTK_OBJECT(servant->appwidget),
 			applet_widget_signals[DO_DRAW_SIGNAL]);
 }
 
 static void
-server_applet_set_tooltips_state(CustomAppletServant *servant,
-				 CORBA_boolean enabled,
+server_applet_set_tooltips_state(PortableServer_Servant _servant,
+				 const CORBA_boolean enabled,
 				 CORBA_Environment *ev)
 {
+	CustomAppletServant *servant = (CustomAppletServant *)_servant;
 	if(!applet_tooltips)
 		applet_tooltips = gtk_tooltips_new();
 
@@ -1377,11 +1556,12 @@ server_applet_set_tooltips_state(CustomAppletServant *servant,
 }
 
 static void
-server_applet_change_position(CustomAppletServant *servant,
+server_applet_change_position(PortableServer_Servant _servant,
 			      CORBA_short x,
 			      CORBA_short y,
 			      CORBA_Environment *ev)
 {
+	CustomAppletServant *servant = (CustomAppletServant *)_servant;
 	if(servant->appwidget->frozen_level>0) {
 		servant->appwidget->frozen_got_position = TRUE;
 		servant->appwidget->frozen_position_x = x;
@@ -1395,23 +1575,26 @@ server_applet_change_position(CustomAppletServant *servant,
 }
 
 static CORBA_char *
-server_applet__get_goad_id(CustomAppletServant *servant,
+server_applet__get_goad_id(PortableServer_Servant _servant,
 			   CORBA_Environment *ev)
 {
+	CustomAppletServant *servant = (CustomAppletServant *)_servant;
 	return CORBA_string_dup(servant->goad_id);
 }
 
 static void
-server_applet_freeze_changes(CustomAppletServant *servant,
+server_applet_freeze_changes(PortableServer_Servant _servant,
 			     CORBA_Environment *ev)
 {
+	CustomAppletServant *servant = (CustomAppletServant *)_servant;
 	servant->appwidget->frozen_level++;
 }
 
 static void
-server_applet_thaw_changes(CustomAppletServant *servant,
+server_applet_thaw_changes(PortableServer_Servant _servant,
 			   CORBA_Environment *ev)
 {
+	CustomAppletServant *servant = (CustomAppletServant *)_servant;
 	if(servant->appwidget->frozen_level>0)
 		servant->appwidget->frozen_level--;
 	
@@ -1426,29 +1609,6 @@ server_applet_thaw_changes(CustomAppletServant *servant,
 	}
 	if(servant->appwidget->frozen_got_size) {
 		servant->appwidget->frozen_got_size = FALSE;
-		switch(servant->appwidget->frozen_size) {
-		case 24:
-			gtk_signal_emit(GTK_OBJECT(servant->appwidget),
-					applet_widget_signals[CHANGE_SIZE_SIGNAL],
-					GNOME_Panel_SIZE_TINY);
-			break;
-		default:
-		case 48:
-			gtk_signal_emit(GTK_OBJECT(servant->appwidget),
-					applet_widget_signals[CHANGE_SIZE_SIGNAL],
-					GNOME_Panel_SIZE_STANDARD);
-			break;
-		case 64:
-			gtk_signal_emit(GTK_OBJECT(servant->appwidget),
-					applet_widget_signals[CHANGE_SIZE_SIGNAL],
-					GNOME_Panel_SIZE_LARGE);
-			break;
-		case 80:
-			gtk_signal_emit(GTK_OBJECT(servant->appwidget),
-					applet_widget_signals[CHANGE_SIZE_SIGNAL],
-					GNOME_Panel_SIZE_HUGE);
-			break;
-		}
 		gtk_signal_emit(GTK_OBJECT(servant->appwidget),
 				applet_widget_signals[CHANGE_PIXEL_SIZE_SIGNAL],
 				servant->appwidget->frozen_size);
@@ -1509,6 +1669,13 @@ orb_remove_connection(GIOPConnection *cnx)
 */
 
 /* Used by shlib applets */
+/**
+ * applet_widget_corba_activate:
+ *
+ * Description:
+ *
+ * Returns:
+ **/
 CORBA_Object
 applet_widget_corba_activate(GtkWidget *applet,
 			     PortableServer_POA poa,
@@ -1520,6 +1687,13 @@ applet_widget_corba_activate(GtkWidget *applet,
 	return CORBA_Object_duplicate(CD(applet)->obj, ev);
 }
 
+/**
+ * applet_widget_corba_deactivate:
+ *
+ * Description:
+ *
+ * Returns:
+ **/
 void
 applet_widget_corba_deactivate(PortableServer_POA poa,
 			       const char *goad_id,
@@ -1538,10 +1712,11 @@ typedef struct {
 } AppletFactory;
 
 static CORBA_boolean
-server_applet_factory_supports(AppletFactory *servant,
+server_applet_factory_supports(PortableServer_Servant _servant,
 			       CORBA_char * obj_goad_id,
 			       CORBA_Environment * ev)
 {
+	AppletFactory *servant = (AppletFactory *)_servant;
 	if(servant->qfunc)
 		return servant->qfunc(obj_goad_id);
 
@@ -1552,11 +1727,12 @@ server_applet_factory_supports(AppletFactory *servant,
 }
 
 static CORBA_Object
-server_applet_factory_create_object(AppletFactory *servant,
+server_applet_factory_create_object(PortableServer_Servant _servant,
 				    CORBA_char * goad_id,
 				    GNOME_stringlist * params,
 				    CORBA_Environment * ev)
 {
+	AppletFactory *servant = (AppletFactory *)_servant;
 	GtkWidget *applet;
 
 	applet = servant->afunc(goad_id, (const char **)params->_buffer,
@@ -1572,8 +1748,8 @@ server_applet_factory_create_object(AppletFactory *servant,
 
 static POA_GNOME_GenericFactory__epv applet_factory_epv = {
 	NULL,
-	(gpointer)&server_applet_factory_supports,
-	(gpointer)&server_applet_factory_create_object,  
+	server_applet_factory_supports,
+	server_applet_factory_create_object,  
 };
 
 static POA_GNOME_GenericFactory__vepv applet_factory_vepv = {
@@ -1581,6 +1757,13 @@ static POA_GNOME_GenericFactory__vepv applet_factory_vepv = {
 	&applet_factory_epv
 };
 
+/**
+ * applet_factory_new:
+ *
+ * Description:
+ *
+ * Returns:
+ **/
 void applet_factory_new(const char *goad_id, AppletFactoryQuerier qfunc,
 			AppletFactoryActivator afunc)
 {
