@@ -15,9 +15,9 @@
 
 #define HAVE_SAVE_SESSION_SIGNAL 1
 #define HAVE_APPLET_BIND_EVENTS 1
-#define HAVE_PANEL_SIZE 1
 #define HAVE_PANEL_PIXEL_SIZE 1
 #define HAVE_PANEL_DRAW_SIGNAL 1
+#define HAVE_APPLET_QUEUE_RESIZE 1
 
 BEGIN_GNOME_DECLS
 
@@ -27,12 +27,6 @@ typedef GNOME_Panel_OrientType PanelOrientType;
 #define ORIENT_LEFT GNOME_Panel_ORIENT_LEFT
 #define ORIENT_RIGHT GNOME_Panel_ORIENT_RIGHT
 
-typedef GNOME_Panel_SizeType PanelSizeType;
-#define SIZE_TINY GNOME_Panel_SIZE_TINY
-#define SIZE_STANDARD GNOME_Panel_SIZE_STANDARD
-#define SIZE_LARGE GNOME_Panel_SIZE_LARGE
-#define SIZE_HUGE GNOME_Panel_SIZE_HUGE
-
 enum {
 	PIXEL_SIZE_TINY = 24,
 	PIXEL_SIZE_SMALL = 36,
@@ -60,17 +54,17 @@ struct _AppletWidget
 {
 	GtkPlug			window;
 	
-	/* this is the public data */
-	
+	/*< public >*/
 	char			*privcfgpath;
 	char			*globcfgpath;
 	
+	/* you should really use the accessors for these anyway */
 	PanelOrientType		orient;			
 	int			size;			
 	
-	/* below this is private data */
-
-        gpointer                corbadat; /* CORBA stuff */
+	/*< private >*/
+	/* CORBA stuff */
+        gpointer                corbadat;
 	
 	/*change freezing*/
 	int			frozen_level;
@@ -101,9 +95,6 @@ struct _AppletWidgetClass
 	   that you can update your orientation properly */
 	void (* change_orient) (AppletWidget *applet,
 				GNOME_Panel_OrientType orient);
-	/* DEPRECATED: when the panel size changes, semantics are the same as above */
-	void (* change_size) (AppletWidget *applet,
-			      GNOME_Panel_SizeType size);
 	/* the panel background changes, the pixmap handeling is likely
 	   to change */
 	void (* back_change) (AppletWidget *applet,
@@ -120,9 +111,9 @@ struct _AppletWidgetClass
 	  gnome_config_drop_all() after your done otherwise the changes
 	  might not be written to file, also make sure you return
 	  FALSE from this signal or your position wil not get saved!*/
-	int (* save_session) (AppletWidget *applet,
-			      char *cfgpath,
-			      char *globcfgpath);
+	gboolean (* save_session) (AppletWidget *applet,
+				   char *cfgpath,
+				   char *globcfgpath);
 	/*when the position changes and we selected to get this signal,
 	  it is sent so that you can move some external window along with
 	  the applet, it is not normally sent, so you need to enable it
@@ -177,7 +168,7 @@ void		applet_widget_add		(AppletWidget *applet,
    just for very special cases*/
 void		applet_widget_add_full		(AppletWidget *applet,
 						 GtkWidget *widget,
-						 int bind_events);
+						 gboolean bind_events);
 
 /* bind the events for button2 and button3 on a widget, this is useful
    when you are added a new widget and want the right click menu and middle
@@ -239,25 +230,22 @@ int		applet_widget_get_applet_count	(void);
 void		applet_widget_sync_config	(AppletWidget *applet);
 
 /* Get the orientation the applet should use */
-GNOME_Panel_OrientType	applet_widget_get_panel_orient	(AppletWidget *applet);
+PanelOrientType	applet_widget_get_panel_orient	(AppletWidget *applet);
 
 /* Get the pixel size the applet should use */
 int		applet_widget_get_panel_pixel_size	(AppletWidget *applet);
 
-/* DEPRECATED: Get the size the applet should use */
-GNOME_Panel_SizeType	applet_widget_get_panel_size	(AppletWidget *applet);
-
 /* Get the free space for the applet if it's on an edge panel or 0
    if on a packed panel or on error */
 int		applet_widget_get_free_space	(AppletWidget *applet);
 
 /* sets if the change_position signal is sent*/
 void		applet_widget_send_position	(AppletWidget *applet,
-						 int enable);
+						 gboolean enable);
 
 /* sets if the do_draw signal is sent*/
 void		applet_widget_send_draw		(AppletWidget *applet,
-						 int enable);
+						 gboolean enable);
 
 /* gets the rgb background, useful in conjunction with the do_draw signal */
 void		applet_widget_get_rgb_bg	(AppletWidget *applet,
@@ -265,10 +253,12 @@ void		applet_widget_get_rgb_bg	(AppletWidget *applet,
 						 int *w, int *h,
 						 int *rowstride);
 
-/*use this instead of gnome init, if you want multi applet, you also
-  have to specify a "start new applet" function which will launch a new
-  applet*/
-int		applet_widget_init		(const char *app_id,
+/* queue resize on the socket in the panel for shlib applets or
+   just the applet for external applets */
+void		applet_widget_queue_resize	(AppletWidget *applet);
+
+/*use this instead of gnome init*/
+gboolean	applet_widget_init		(const char *app_id,
 						 const char *app_version,
 						 int argc,
 						 char **argv,
