@@ -25,14 +25,6 @@ static void basep_widget_init		(BasePWidget      *basep);
 static void basep_pos_class_init (BasePPosClass *klass);
 static void basep_pos_init (BasePPos *pos);
 
-static int
-basep_drag_motion (PanelWidget        *panel,
-		   GdkDragContext     *context,
-		   gint                x,
-		   gint                y,
-		   guint               time,
-		   BasePWidget        *basep);
-
 static GtkWindowClass *basep_widget_parent_class = NULL;
 static GtkObjectClass *basep_pos_parent_class = NULL;
 /*global settings*/
@@ -443,8 +435,7 @@ basep_enter_notify(BasePWidget *basep,
 		basep_widget_autoshow (basep);
 	}  
 
-	if (!xstuff_is_compliant_wm() &&
-	    global_config.autoraise)
+	if (global_config.autoraise)
 		gdk_window_raise(GTK_WIDGET(basep)->window);
 
 	return FALSE;
@@ -606,7 +597,7 @@ basep_widget_do_hiding(BasePWidget *basep, PanelOrientType hide_orient,
 			(diff/1000.0)*200*(10001-(step*step));
 
 		basep_widget_set_ebox_orient(basep, hide_orient);
-		
+
 		while(x != dx ||
 		      y != dy ||
 		      w != dw ||
@@ -1098,14 +1089,14 @@ basep_widget_west_clicked (GtkWidget *widget, gpointer data)
 
 GtkWidget*
 basep_widget_construct (BasePWidget *basep,
-			int packed,
-			int reverse_arrows,
+			gboolean packed,
+			gboolean reverse_arrows,
 			PanelOrientation orient,
 			int sz,
 			BasePMode mode,
 			BasePState state,
-			int hidebuttons_enabled,
-			int hidebutton_pixmaps_enabled,
+			gboolean hidebuttons_enabled,
+			gboolean hidebutton_pixmaps_enabled,
 			PanelBackType back_type,
 			char *back_pixmap,
 			gboolean fit_pixmap_bg,
@@ -1114,7 +1105,7 @@ basep_widget_construct (BasePWidget *basep,
 			GdkColor *back_color)
 {
 	BasePPosClass *klass = basep_widget_get_pos_class (basep);
-	int x=0, y=0;
+	int x = 0, y = 0;
 	basep->panel = panel_widget_new(packed,
 					orient,
 					sz,
@@ -1123,6 +1114,9 @@ basep_widget_construct (BasePWidget *basep,
 					fit_pixmap_bg,
 					strech_pixmap_bg,
 					rotate_pixmap_bg,
+					/*if hidebuttons are enabled, then
+					  do no padding on the sides */
+					!hidebuttons_enabled,
 					back_color);
 
 	gtk_signal_connect_after(GTK_OBJECT(basep->panel), "back_change",
@@ -1188,10 +1182,6 @@ basep_widget_construct (BasePWidget *basep,
 			    GTK_SIGNAL_FUNC (basep_widget_south_clicked),
 			    basep);
 
-	gtk_signal_connect (GTK_OBJECT (basep->panel), "drag_motion",
-			    GTK_SIGNAL_FUNC (basep_drag_motion),
-			    basep);
-
 	basep->hidebuttons_enabled = hidebuttons_enabled;
 	basep->hidebutton_pixmaps_enabled = hidebutton_pixmaps_enabled;
 
@@ -1222,8 +1212,8 @@ basep_widget_change_params(BasePWidget *basep,
 			   int sz,
 			   BasePMode mode,
 			   BasePState state,
-			   int hidebuttons_enabled,
-			   int hidebutton_pixmaps_enabled,
+			   gboolean hidebuttons_enabled,
+			   gboolean hidebutton_pixmaps_enabled,
 			   PanelBackType back_type,
 			   char *pixmap_name,
 			   gboolean fit_pixmap_bg,
@@ -1275,6 +1265,9 @@ basep_widget_change_params(BasePWidget *basep,
 				   fit_pixmap_bg,
 				   strech_pixmap_bg,
 				   rotate_pixmap_bg,
+				   /*if hidebuttons are enabled, then
+				     do no padding on the sides */
+				   !hidebuttons_enabled,
 				   back_color);
 
 	basep_widget_set_hidebuttons(basep);
@@ -1380,6 +1373,13 @@ basep_widget_explicit_hide (BasePWidget *basep, BasePState state)
 	}
 
 	gnome_triggers_vdo("", NULL, supinfo);
+
+	gtk_signal_emit(GTK_OBJECT(basep),
+			basep_widget_signals[STATE_CHANGE_SIGNAL],
+			state);
+
+	/* if the app did any updating of the interface, flush that for us*/
+	gdk_flush();
 	
 	if (GTK_WIDGET_REALIZED(GTK_WIDGET(basep))) {
 		BasePPosClass *klass = basep_widget_get_pos_class (basep);
@@ -1405,11 +1405,6 @@ basep_widget_explicit_hide (BasePWidget *basep, BasePState state)
 
 	basep->state = state;
 	basep_widget_update_winhints (basep);
-
-	gtk_signal_emit(GTK_OBJECT(basep),
-			basep_widget_signals[STATE_CHANGE_SIGNAL],
-			basep->state);
-
 }
 
 void
@@ -1545,6 +1540,13 @@ basep_widget_autohide (gpointer data)
 
 	gnome_triggers_vdo("", NULL, supinfo);
 
+	gtk_signal_emit(GTK_OBJECT(basep),
+			basep_widget_signals[STATE_CHANGE_SIGNAL],
+			BASEP_AUTO_HIDDEN);
+
+	/* if the app did any updating of the interface, flush that for us*/
+	gdk_flush();
+
 	if (GTK_WIDGET_REALIZED(basep)) {
 		BasePPosClass *klass = basep_widget_get_pos_class (basep);
 		PanelOrientType hide_orient;
@@ -1573,10 +1575,6 @@ basep_widget_autohide (gpointer data)
 	basep->state = BASEP_AUTO_HIDDEN;
 	basep_widget_update_winhints (basep);
 
-	gtk_signal_emit(GTK_OBJECT(basep),
-			basep_widget_signals[STATE_CHANGE_SIGNAL],
-			BASEP_AUTO_HIDDEN);
-
 	basep->leave_notify_timer_tag = 0;
 	return FALSE;
 }
@@ -1612,32 +1610,6 @@ basep_widget_queue_autohide(BasePWidget *basep)
                                  basep_widget_autohide, basep);
 }
 
-static int
-basep_drag_motion (PanelWidget        *panel,
-		   GdkDragContext     *context,
-		   gint                x,
-		   gint                y,
-		   guint               time,
-		   BasePWidget        *basep)
-{
-	gdk_drag_status (context, context->suggested_action, time);
-
-        if ((basep->mode == BASEP_EXPLICIT_HIDE) ||
-	    (basep->state == BASEP_HIDDEN_LEFT) ||
-	    (basep->state == BASEP_HIDDEN_RIGHT))
-	        return TRUE;
-
-	if (basep->leave_notify_timer_tag != 0) {
-	        gtk_timeout_remove (basep->leave_notify_timer_tag);
-		basep->leave_notify_timer_tag = 0;
-	}
-
-	basep_widget_autoshow(basep);
-	basep_widget_queue_autohide (basep);
-
-	return TRUE;
-}
-
 void
 basep_widget_get_menu_pos (BasePWidget *basep,
 			   GtkWidget *menu,
