@@ -128,20 +128,6 @@ drawer_pos_set_hidebuttons (BasePWidget *basep)
 	}
 }
 
-static void
-get_widget_pos(GtkWidget *widget, int *x, int *y)
-{
-       if(!widget->window) {
-               *x = *y = 0;
-               return;
-       }
-       gdk_window_get_origin (widget->window, x, y);
-       if(GTK_WIDGET_NO_WINDOW(widget)) {
-               *x += widget->allocation.x;
-               *y += widget->allocation.y;
-       }
-}
-
 static PanelOrientType
 drawer_pos_get_applet_orient (BasePWidget *basep)
 {
@@ -149,7 +135,8 @@ drawer_pos_get_applet_orient (BasePWidget *basep)
 	PanelOrientation porient = panel->orient;
 	int x,y;
 
-	get_widget_pos(GTK_WIDGET(basep),&x,&y);
+	x = GTK_WIDGET(basep)->allocation.x;
+	y = GTK_WIDGET(basep)->allocation.y;
 
 	if(porient == PANEL_VERTICAL) {
 		if(x > (gdk_screen_width()/2))
@@ -206,6 +193,10 @@ void
 drawer_widget_close_drawer (DrawerWidget *drawer, BasePWidget *parentp)
 {
 	BasePWidget *basep = BASEP_WIDGET (drawer);
+
+	if(GTK_WIDGET(parentp)->window)
+		gdk_window_raise(GTK_WIDGET(parentp)->window);
+
 	switch (DRAWER_POS (basep->pos)->orient) {
 	case ORIENT_UP:
 	case ORIENT_LEFT:
@@ -260,6 +251,14 @@ drawer_pos_get_pos(BasePWidget *basep,
 {
 	PanelWidget *panel = PANEL_WIDGET(basep->panel);
 	DrawerPos *pos = DRAWER_POS (basep->pos);
+
+	/* we are shown but we are hidden, life is full of ironies */
+	if (pos->temp_hidden) {
+		*x = -width - 1;
+		*y = -height - 1;
+		return;
+	}
+
 	if (panel->master_widget &&
 	    GTK_WIDGET_REALIZED (panel->master_widget) &&
 	    /*"allocated" data will be set on each allocation, until then,
