@@ -54,139 +54,139 @@ PortableServer_POA thepoa;
 
 /***Panel stuff***/
 static GNOME_PanelSpot
-s_panel_add_applet(POA_GNOME_Panel *servant,
-		   GNOME_Applet panel_applet,
-		   CORBA_char *goad_id,
+s_panel_add_applet(PortableServer_Servant servant,
+		   const GNOME_Applet panel_applet,
+		   const CORBA_char *goad_id,
 		   CORBA_char ** cfgpath,
 		   CORBA_char ** globcfgpath,
 		   CORBA_unsigned_long* wid,
 		   CORBA_Environment *ev);
 
 static GNOME_PanelSpot
-s_panel_add_applet_full(POA_GNOME_Panel *servant,
-			GNOME_Applet panel_applet,
-			CORBA_char *goad_id,
-			CORBA_short panel,
-			CORBA_short pos,
+s_panel_add_applet_full(PortableServer_Servant servant,
+			const GNOME_Applet panel_applet,
+			const CORBA_char *goad_id,
+			const CORBA_short panel,
+			const CORBA_short pos,
 			CORBA_char ** cfgpath,
 			CORBA_char ** globcfgpath,
 			CORBA_unsigned_long* wid,
 			CORBA_Environment *ev);
 
 static void
-s_panel_quit(POA_GNOME_Panel *servant, CORBA_Environment *ev);
+s_panel_quit(PortableServer_Servant servant, CORBA_Environment *ev);
 
 static CORBA_boolean
-s_panel_get_in_drag(POA_GNOME_Panel *servant, CORBA_Environment *ev);
+s_panel_get_in_drag(PortableServer_Servant servant, CORBA_Environment *ev);
 
 static GNOME_StatusSpot
-s_panel_add_status(POA_GNOME_Panel *servant,
+s_panel_add_status(PortableServer_Servant servant,
 		   CORBA_unsigned_long* wid,
 		   CORBA_Environment *ev);
 
 static void
-s_panel_notice_config_changes(POA_GNOME_Panel *servant,
+s_panel_notice_config_changes(PortableServer_Servant servant,
 			      CORBA_Environment *ev);
 
 
 /*** PanelSpot stuff ***/
 
 static CORBA_char *
-s_panelspot_get_tooltip(POA_GNOME_PanelSpot *servant,
+s_panelspot_get_tooltip(PortableServer_Servant servant,
 			CORBA_Environment *ev);
 
 static void
-s_panelspot_set_tooltip(POA_GNOME_PanelSpot *servant,
-			CORBA_char *val,
+s_panelspot_set_tooltip(PortableServer_Servant servant,
+			const CORBA_char *val,
 			CORBA_Environment *ev);
 
 static CORBA_short
-s_panelspot_get_parent_panel(POA_GNOME_PanelSpot *servant,
+s_panelspot_get_parent_panel(PortableServer_Servant servant,
 			     CORBA_Environment *ev);
 
 static CORBA_short
-s_panelspot_get_spot_pos(POA_GNOME_PanelSpot *servant,
+s_panelspot_get_spot_pos(PortableServer_Servant servant,
 			 CORBA_Environment *ev);
 
 static GNOME_Panel_OrientType
-s_panelspot_get_parent_orient(POA_GNOME_PanelSpot *servant,
+s_panelspot_get_parent_orient(PortableServer_Servant servant,
 			      CORBA_Environment *ev);
 
-static GNOME_Panel_SizeType
-s_panelspot_get_parent_size(POA_GNOME_PanelSpot *servant,
+static CORBA_short
+s_panelspot_get_parent_size(PortableServer_Servant servant,
 			    CORBA_Environment *ev);
 
 static CORBA_short
-s_panelspot_get_free_space(POA_GNOME_PanelSpot *servant,
+s_panelspot_get_free_space(PortableServer_Servant servant,
 			   CORBA_Environment *ev);
 
 static CORBA_boolean
-s_panelspot_get_send_position(POA_GNOME_PanelSpot *servant,
+s_panelspot_get_send_position(PortableServer_Servant servant,
 			      CORBA_Environment *ev);
 static void
-s_panelspot_set_send_position(POA_GNOME_PanelSpot *servant,
+s_panelspot_set_send_position(PortableServer_Servant servant,
 			      CORBA_boolean,
 			      CORBA_Environment *ev);
 
 static CORBA_boolean
-s_panelspot_get_send_draw(POA_GNOME_PanelSpot *servant,
+s_panelspot_get_send_draw(PortableServer_Servant servant,
 			  CORBA_Environment *ev);
 static void
-s_panelspot_set_send_draw(POA_GNOME_PanelSpot *servant,
+s_panelspot_set_send_draw(PortableServer_Servant servant,
 			  CORBA_boolean,
 			  CORBA_Environment *ev);
 
 static GNOME_Panel_RgbImage *
-s_panelspot_get_rgb_background(POA_GNOME_PanelSpot *servant,
+s_panelspot_get_rgb_background(PortableServer_Servant servant,
 			       CORBA_Environment *ev);
 
 static void
-s_panelspot_register_us(POA_GNOME_PanelSpot *servant,
+s_panelspot_register_us(PortableServer_Servant servant,
 		     CORBA_Environment *ev);
 
 static void
-s_panelspot_unregister_us(POA_GNOME_PanelSpot *servant,
+s_panelspot_unregister_us(PortableServer_Servant servant,
 		       CORBA_Environment *ev);
 
 static void
-s_panelspot_abort_load(POA_GNOME_PanelSpot *servant,
+s_panelspot_abort_load(PortableServer_Servant servant,
 		       CORBA_Environment *ev);
 
 static void
-s_panelspot_show_menu(POA_GNOME_PanelSpot *servant,
+s_panelspot_show_menu(PortableServer_Servant servant,
 		      CORBA_Environment *ev);
 
 static void
-s_panelspot_drag_start(POA_GNOME_PanelSpot *servant,
+s_panelspot_drag_start(PortableServer_Servant servant,
 		       CORBA_Environment *ev);
 
 static void
-s_panelspot_drag_stop(POA_GNOME_PanelSpot *servant,
+s_panelspot_drag_stop(PortableServer_Servant servant,
 		      CORBA_Environment *ev);
 
 static void
-s_panelspot_add_callback(POA_GNOME_PanelSpot *servant,
-			 CORBA_char *callback_name,
-			 CORBA_char *stock_item,
-			 CORBA_char *menuitem_text,
+s_panelspot_add_callback(PortableServer_Servant servant,
+			 const CORBA_char *callback_name,
+			 const CORBA_char *stock_item,
+			 const CORBA_char *menuitem_text,
 			 CORBA_Environment *ev);
 
 static void
-s_panelspot_remove_callback(POA_GNOME_PanelSpot *servant,
-			    CORBA_char *callback_name,
+s_panelspot_remove_callback(PortableServer_Servant servant,
+			    const CORBA_char *callback_name,
 			    CORBA_Environment *ev);
 static void
-s_panelspot_callback_set_sensitive(POA_GNOME_PanelSpot *servant,
-				   CORBA_char *callback_name,
-				   CORBA_boolean sensitive,
+s_panelspot_callback_set_sensitive(PortableServer_Servant servant,
+				   const CORBA_char *callback_name,
+				   const CORBA_boolean sensitive,
 				   CORBA_Environment *ev);
 
 static void
-s_panelspot_sync_config(POA_GNOME_PanelSpot *servant,
+s_panelspot_sync_config(PortableServer_Servant servant,
 			CORBA_Environment *ev);
 
 static void
-s_panelspot_done_session_save(POA_GNOME_PanelSpot *servant,
+s_panelspot_done_session_save(PortableServer_Servant servant,
 			      CORBA_boolean ret,
 			      CORBA_unsigned_long cookie,
 			      CORBA_Environment *ev);
@@ -208,12 +208,12 @@ static PortableServer_ServantBase__epv panel_base_epv = {
 
 static POA_GNOME_Panel__epv panel_epv = {
   NULL, /* private data */
-  (gpointer)&s_panel_add_applet,
-  (gpointer)&s_panel_add_applet_full,
-  (gpointer)&s_panel_quit,
-  (gpointer)&s_panel_get_in_drag,
-  (gpointer)&s_panel_add_status,
-  (gpointer)&s_panel_notice_config_changes
+  s_panel_add_applet,
+  s_panel_add_applet_full,
+  s_panel_quit,
+  s_panel_get_in_drag,
+  s_panel_add_status,
+  s_panel_notice_config_changes
 };
 static POA_GNOME_Panel__vepv panel_vepv = { &panel_base_epv, &panel_epv };
 static POA_GNOME_Panel panel_servant = { NULL, &panel_vepv };
@@ -227,29 +227,29 @@ static PortableServer_ServantBase__epv panelspot_base_epv = {
 
 static POA_GNOME_PanelSpot__epv panelspot_epv = {
   NULL, /* private data */
-  (gpointer)&s_panelspot_get_tooltip,
-  (gpointer)&s_panelspot_set_tooltip,
-  (gpointer)&s_panelspot_get_parent_panel,
-  (gpointer)&s_panelspot_get_spot_pos,
-  (gpointer)&s_panelspot_get_parent_orient,
-  (gpointer)&s_panelspot_get_parent_size,
-  (gpointer)&s_panelspot_get_free_space,
-  (gpointer)&s_panelspot_get_send_position,
-  (gpointer)&s_panelspot_set_send_position,
-  (gpointer)&s_panelspot_get_send_draw,
-  (gpointer)&s_panelspot_set_send_draw,
-  (gpointer)&s_panelspot_get_rgb_background,
-  (gpointer)&s_panelspot_register_us,
-  (gpointer)&s_panelspot_unregister_us,
-  (gpointer)&s_panelspot_abort_load,
-  (gpointer)&s_panelspot_show_menu,
-  (gpointer)&s_panelspot_drag_start,
-  (gpointer)&s_panelspot_drag_stop,
-  (gpointer)&s_panelspot_add_callback,
-  (gpointer)&s_panelspot_remove_callback,
-  (gpointer)&s_panelspot_callback_set_sensitive,
-  (gpointer)&s_panelspot_sync_config,
-  (gpointer)&s_panelspot_done_session_save
+  s_panelspot_get_tooltip,
+  s_panelspot_set_tooltip,
+  s_panelspot_get_parent_panel,
+  s_panelspot_get_spot_pos,
+  s_panelspot_get_parent_orient,
+  s_panelspot_get_parent_size,
+  s_panelspot_get_free_space,
+  s_panelspot_get_send_position,
+  s_panelspot_set_send_position,
+  s_panelspot_get_send_draw,
+  s_panelspot_set_send_draw,
+  s_panelspot_get_rgb_background,
+  s_panelspot_register_us,
+  s_panelspot_unregister_us,
+  s_panelspot_abort_load,
+  s_panelspot_show_menu,
+  s_panelspot_drag_start,
+  s_panelspot_drag_stop,
+  s_panelspot_add_callback,
+  s_panelspot_remove_callback,
+  s_panelspot_callback_set_sensitive,
+  s_panelspot_sync_config,
+  s_panelspot_done_session_save
 };
 static POA_GNOME_PanelSpot__vepv panelspot_vepv = { &panelspot_base_epv, &panelspot_epv };
 
@@ -306,7 +306,31 @@ extern_clean(Extern *ext)
 static void
 extern_socket_destroy(GtkWidget *w, gpointer data)
 {
+	GtkSocket *socket = GTK_SOCKET(w);
 	Extern *ext = data;
+
+	if (socket->same_app && socket->plug_window) {
+		GtkWidget *plug_widget;
+		gdk_window_get_user_data (socket->plug_window,
+					  (gpointer *)&plug_widget);
+		if(plug_widget) {
+			/* XXX: hackaround to broken gtkplug/gtksocket!!!
+			   KILL all references to ourselves on the plug
+			   and all our references to the plug and then
+			   destroy it.*/
+			GtkWidget *toplevel = gtk_widget_get_toplevel (w);
+			if (toplevel && GTK_IS_WINDOW (toplevel))
+				gtk_window_remove_embedded_xid (GTK_WINDOW (toplevel), 
+								GDK_WINDOW_XWINDOW (socket->plug_window));
+
+			socket->plug_window = NULL;
+			socket->same_app = FALSE;
+			GTK_PLUG(plug_widget)->socket_window = NULL;
+			GTK_PLUG(plug_widget)->same_app = FALSE;
+			gtk_widget_destroy(plug_widget);
+		}
+	}
+
 	gtk_widget_destroy(ext->ebox);
 	extern_clean(ext);
 }
@@ -548,9 +572,9 @@ load_queued_externs(void)
 
 
 static GNOME_PanelSpot
-s_panel_add_applet(POA_GNOME_Panel *servant,
-		   GNOME_Applet panel_applet,
-		   CORBA_char *goad_id,
+s_panel_add_applet(PortableServer_Servant servant,
+		   const GNOME_Applet panel_applet,
+		   const CORBA_char *goad_id,
 		   CORBA_char ** cfgpath,
 		   CORBA_char ** globcfgpath,
 		   CORBA_unsigned_long* wid,
@@ -561,11 +585,11 @@ s_panel_add_applet(POA_GNOME_Panel *servant,
 }
 
 static GNOME_PanelSpot
-s_panel_add_applet_full(POA_GNOME_Panel *servant,
-			GNOME_Applet panel_applet,
-			CORBA_char *goad_id,
-			CORBA_short panel,
-			CORBA_short pos,
+s_panel_add_applet_full(PortableServer_Servant servant,
+			const GNOME_Applet panel_applet,
+			const CORBA_char *goad_id,
+			const CORBA_short panel,
+			const CORBA_short pos,
 			CORBA_char ** cfgpath,
 			CORBA_char ** globcfgpath,
 			CORBA_unsigned_long* wid,
@@ -573,7 +597,6 @@ s_panel_add_applet_full(POA_GNOME_Panel *servant,
 {
 	GSList *li;
 	Extern *ext;
-	char *p;
 	POA_GNOME_PanelSpot *panelspot_servant;
 	GNOME_PanelSpot acc;
 	
@@ -663,28 +686,26 @@ s_panel_add_applet_full(POA_GNOME_Panel *servant,
 		*cfgpath = NULL;
 		return CORBA_OBJECT_NIL;
 	}
-	p = g_strconcat(PANEL_CONFIG_PATH,"Applet_Dummy/",NULL);
-	*cfgpath = CORBA_string_dup(p);
-	g_free(p);
+	*cfgpath = CORBA_string_dup(PANEL_CONFIG_PATH "Applet_Dummy/");
 	*globcfgpath = CORBA_string_dup (PANEL_CONFIG_PATH);
 
 	return CORBA_Object_duplicate(acc, ev);
 }
 
 static void
-s_panel_quit(POA_GNOME_Panel *servant, CORBA_Environment *ev)
+s_panel_quit(PortableServer_Servant servant, CORBA_Environment *ev)
 {
 	panel_quit();
 }
 
 static CORBA_boolean
-s_panel_get_in_drag(POA_GNOME_Panel *servant, CORBA_Environment *ev)
+s_panel_get_in_drag(PortableServer_Servant servant, CORBA_Environment *ev)
 {
 	return panel_applet_in_drag;
 }
 
 static GNOME_StatusSpot
-s_panel_add_status(POA_GNOME_Panel *servant,
+s_panel_add_status(PortableServer_Servant servant,
 		   CORBA_unsigned_long *wid,
 		   CORBA_Environment *ev)
 {
@@ -722,7 +743,7 @@ s_panel_add_status(POA_GNOME_Panel *servant,
 }
 
 static void
-s_panel_notice_config_changes(POA_GNOME_Panel *servant,
+s_panel_notice_config_changes(PortableServer_Servant servant,
 			      CORBA_Environment *ev)
 {
 	load_up_globals();
@@ -732,7 +753,7 @@ s_panel_notice_config_changes(POA_GNOME_Panel *servant,
 /*** PanelSpot stuff ***/
 
 static CORBA_char *
-s_panelspot_get_tooltip(POA_GNOME_PanelSpot *servant,
+s_panelspot_get_tooltip(PortableServer_Servant servant,
 			CORBA_Environment *ev)
 {
 	Extern *ext = (Extern *)servant;
@@ -744,19 +765,19 @@ s_panelspot_get_tooltip(POA_GNOME_PanelSpot *servant,
 }
 
 static void
-s_panelspot_set_tooltip(POA_GNOME_PanelSpot *servant,
-			CORBA_char *val,
+s_panelspot_set_tooltip(PortableServer_Servant servant,
+			const CORBA_char *val,
 			CORBA_Environment *ev)
 {
 	Extern *ext = (Extern *)servant;
 	if(val && *val)
-		gtk_tooltips_set_tip (panel_tooltips,ext->ebox,val,NULL);
+		gtk_tooltips_set_tip (panel_tooltips, ext->ebox, val, NULL);
 	else
-		gtk_tooltips_set_tip (panel_tooltips,ext->ebox,NULL,NULL);
+		gtk_tooltips_set_tip (panel_tooltips, ext->ebox, NULL, NULL);
 }
 
 static CORBA_short
-s_panelspot_get_parent_panel(POA_GNOME_PanelSpot *servant,
+s_panelspot_get_parent_panel(PortableServer_Servant servant,
 			     CORBA_Environment *ev)
 {
 	int panel;
@@ -776,7 +797,7 @@ s_panelspot_get_parent_panel(POA_GNOME_PanelSpot *servant,
 }
 
 static CORBA_short
-s_panelspot_get_spot_pos(POA_GNOME_PanelSpot *servant,
+s_panelspot_get_spot_pos(PortableServer_Servant servant,
 			 CORBA_Environment *ev)
 {
 	Extern *ext = (Extern *)servant;
@@ -793,7 +814,7 @@ s_panelspot_get_spot_pos(POA_GNOME_PanelSpot *servant,
 }
 
 static GNOME_Panel_OrientType
-s_panelspot_get_parent_orient(POA_GNOME_PanelSpot *servant,
+s_panelspot_get_parent_orient(PortableServer_Servant servant,
 			      CORBA_Environment *ev)
 {
 	Extern *ext = (Extern *)servant;
@@ -813,8 +834,8 @@ s_panelspot_get_parent_orient(POA_GNOME_PanelSpot *servant,
 	return get_applet_orient(panel);
 }
 
-static GNOME_Panel_SizeType
-s_panelspot_get_parent_size(POA_GNOME_PanelSpot *servant,
+static CORBA_short
+s_panelspot_get_parent_size(PortableServer_Servant servant,
 			    CORBA_Environment *ev)
 {
 	Extern *ext = (Extern *)servant;
@@ -835,7 +856,7 @@ s_panelspot_get_parent_size(POA_GNOME_PanelSpot *servant,
 }
 
 static CORBA_short
-s_panelspot_get_free_space(POA_GNOME_PanelSpot *servant,
+s_panelspot_get_free_space(PortableServer_Servant servant,
 			   CORBA_Environment *ev)
 {
 	Extern *ext = (Extern *)servant;
@@ -856,7 +877,7 @@ s_panelspot_get_free_space(POA_GNOME_PanelSpot *servant,
 }
 
 static CORBA_boolean
-s_panelspot_get_send_position(POA_GNOME_PanelSpot *servant,
+s_panelspot_get_send_position(PortableServer_Servant servant,
 			      CORBA_Environment *ev)
 {
 	Extern *ext = (Extern *)servant;
@@ -867,7 +888,7 @@ s_panelspot_get_send_position(POA_GNOME_PanelSpot *servant,
 }
 
 static void
-s_panelspot_set_send_position(POA_GNOME_PanelSpot *servant,
+s_panelspot_set_send_position(PortableServer_Servant servant,
 			      CORBA_boolean enable,
 			      CORBA_Environment *ev)
 {
@@ -879,7 +900,7 @@ s_panelspot_set_send_position(POA_GNOME_PanelSpot *servant,
 }
 
 static CORBA_boolean
-s_panelspot_get_send_draw(POA_GNOME_PanelSpot *servant,
+s_panelspot_get_send_draw(PortableServer_Servant servant,
 			  CORBA_Environment *ev)
 {
 	Extern *ext = (Extern *)servant;
@@ -890,7 +911,7 @@ s_panelspot_get_send_draw(POA_GNOME_PanelSpot *servant,
 }
 
 static void
-s_panelspot_set_send_draw(POA_GNOME_PanelSpot *servant,
+s_panelspot_set_send_draw(PortableServer_Servant servant,
 			  CORBA_boolean enable,
 			  CORBA_Environment *ev)
 {
@@ -902,7 +923,7 @@ s_panelspot_set_send_draw(POA_GNOME_PanelSpot *servant,
 }
 
 static GNOME_Panel_RgbImage *
-s_panelspot_get_rgb_background(POA_GNOME_PanelSpot *servant,
+s_panelspot_get_rgb_background(PortableServer_Servant servant,
 			       CORBA_Environment *ev)
 {
 	Extern *ext = (Extern *)servant;
@@ -947,7 +968,7 @@ s_panelspot_get_rgb_background(POA_GNOME_PanelSpot *servant,
 }
 
 static void
-s_panelspot_register_us(POA_GNOME_PanelSpot *servant,
+s_panelspot_register_us(PortableServer_Servant servant,
 			CORBA_Environment *ev)
 {
 	PanelWidget *panel;
@@ -984,7 +1005,7 @@ s_panelspot_register_us(POA_GNOME_PanelSpot *servant,
 }
 
 static void
-s_panelspot_unregister_us(POA_GNOME_PanelSpot *servant,
+s_panelspot_unregister_us(PortableServer_Servant servant,
 			  CORBA_Environment *ev)
 {
 	Extern *ext = (Extern *)servant;
@@ -992,7 +1013,7 @@ s_panelspot_unregister_us(POA_GNOME_PanelSpot *servant,
 }
 
 static void
-s_panelspot_abort_load(POA_GNOME_PanelSpot *servant,
+s_panelspot_abort_load(PortableServer_Servant servant,
 		       CORBA_Environment *ev)
 {
 	Extern *ext = (Extern *)servant;
@@ -1010,7 +1031,7 @@ s_panelspot_abort_load(POA_GNOME_PanelSpot *servant,
 }
 
 static void
-s_panelspot_show_menu(POA_GNOME_PanelSpot *servant,
+s_panelspot_show_menu(PortableServer_Servant servant,
 		      CORBA_Environment *ev)
 {
 	GtkWidget *panel;
@@ -1040,7 +1061,7 @@ s_panelspot_show_menu(POA_GNOME_PanelSpot *servant,
 
 
 static void
-s_panelspot_drag_start(POA_GNOME_PanelSpot *servant,
+s_panelspot_drag_start(PortableServer_Servant servant,
 		       CORBA_Environment *ev)
 {
 	PanelWidget *panel;
@@ -1062,7 +1083,7 @@ s_panelspot_drag_start(POA_GNOME_PanelSpot *servant,
 }
 
 static void
-s_panelspot_drag_stop(POA_GNOME_PanelSpot *servant,
+s_panelspot_drag_stop(PortableServer_Servant servant,
 		      CORBA_Environment *ev)
 {
 	PanelWidget *panel;
@@ -1082,10 +1103,10 @@ s_panelspot_drag_stop(POA_GNOME_PanelSpot *servant,
 }
 
 static void
-s_panelspot_add_callback(POA_GNOME_PanelSpot *servant,
-			 CORBA_char *callback_name,
-			 CORBA_char *stock_item,
-			 CORBA_char *menuitem_text,
+s_panelspot_add_callback(PortableServer_Servant servant,
+			 const CORBA_char *callback_name,
+			 const CORBA_char *stock_item,
+			 const CORBA_char *menuitem_text,
 			 CORBA_Environment *ev)
 {
 	Extern *ext = (Extern *)servant;
@@ -1101,8 +1122,8 @@ s_panelspot_add_callback(POA_GNOME_PanelSpot *servant,
 }
 
 static void
-s_panelspot_remove_callback(POA_GNOME_PanelSpot *servant,
-			    CORBA_char *callback_name,
+s_panelspot_remove_callback(PortableServer_Servant servant,
+			    const CORBA_char *callback_name,
 			    CORBA_Environment *ev)
 {
 	Extern *ext = (Extern *)servant;
@@ -1113,9 +1134,9 @@ s_panelspot_remove_callback(POA_GNOME_PanelSpot *servant,
 }
 
 static void
-s_panelspot_callback_set_sensitive(POA_GNOME_PanelSpot *servant,
-				   CORBA_char *callback_name,
-				   CORBA_boolean sensitive,
+s_panelspot_callback_set_sensitive(PortableServer_Servant servant,
+				   const CORBA_char *callback_name,
+				   const CORBA_boolean sensitive,
 				   CORBA_Environment *ev)
 {
 	Extern *ext = (Extern *)servant;
@@ -1126,7 +1147,7 @@ s_panelspot_callback_set_sensitive(POA_GNOME_PanelSpot *servant,
 }
 
 static void
-s_panelspot_sync_config(POA_GNOME_PanelSpot *servant,
+s_panelspot_sync_config(PortableServer_Servant servant,
 			CORBA_Environment *ev)
 {
 	Extern *ext = (Extern *)servant;
@@ -1144,57 +1165,15 @@ save_next_idle(gpointer data)
 	return FALSE;
 }
 
-static void
-s_panelspot_done_session_save(POA_GNOME_PanelSpot *servant,
-			      CORBA_boolean ret,
-			      CORBA_unsigned_long cookie,
-			      CORBA_Environment *ev)
+void
+save_applet(AppletInfo *info, gboolean ret)
 {
-	GSList *cur;
-	AppletInfo *info;
 	char *buf;
 	PanelWidget *panel;
 	AppletData *ad;
 	Extern *ext;
 	int panel_num;
 	
-	/*ignore bad cookies*/
-	if(cookie != ss_cookie)
-		return;
-
-	/*increment cookie to kill the timeout warning*/
-	ss_cookie++;
-	
-	if(ss_timeout_dlg) {
-		gtk_widget_destroy(ss_timeout_dlg);
-		ss_timeout_dlg = NULL;
-	}
-
-	if(g_slist_length(applets)<=ss_cur_applet) {
-		ss_done_save = TRUE;
-		return;
-	}
-	
-	cur = g_slist_nth(applets,ss_cur_applet);
-	
-	if(!cur) {
-		ss_done_save = TRUE;
-		return;
-	}
-	
-	info = cur->data;
-
-	/*hmm, this came from a different applet?, we are
-	  getting seriously confused*/
-	if(info->type!=APPLET_EXTERN ||
-	   (gpointer)servant!=(gpointer)info->data) {
-		applets_to_sync = TRUE; /*we need to redo this yet again*/
-		/*save next applet, but from an idle handler, so that
-		  this call returns*/
-		gtk_idle_add(save_next_idle,NULL);
-		return;
-	}
-	
 	ext = info->data;
 	
 	buf = g_strdup_printf("%sApplet_Config/Applet_%d/", PANEL_CONFIG_PATH, info->applet_id+1);
@@ -1242,6 +1221,55 @@ s_panelspot_done_session_save(POA_GNOME_PanelSpot *servant,
 	gtk_idle_add(save_next_idle,NULL);
 }
 
+static void
+s_panelspot_done_session_save(PortableServer_Servant servant,
+			      CORBA_boolean ret,
+			      CORBA_unsigned_long cookie,
+			      CORBA_Environment *ev)
+{
+	GSList *cur;
+	AppletInfo *info;
+	
+	/*ignore bad cookies*/
+	if(cookie != ss_cookie)
+		return;
+
+	/*increment cookie to kill the timeout warning*/
+	ss_cookie++;
+	
+	if(ss_timeout_dlg) {
+		gtk_widget_destroy(ss_timeout_dlg);
+		ss_timeout_dlg = NULL;
+	}
+
+	if(g_slist_length(applets)<=ss_cur_applet) {
+		ss_done_save = TRUE;
+		return;
+	}
+	
+	cur = g_slist_nth(applets,ss_cur_applet);
+	
+	if(!cur) {
+		ss_done_save = TRUE;
+		return;
+	}
+	
+	info = cur->data;
+
+	/*hmm, this came from a different applet?, we are
+	  getting seriously confused*/
+	if(info->type!=APPLET_EXTERN ||
+	   (gpointer)servant!=(gpointer)info->data) {
+		applets_to_sync = TRUE; /*we need to redo this yet again*/
+		/*save next applet, but from an idle handler, so that
+		  this call returns*/
+		gtk_idle_add(save_next_idle,NULL);
+		return;
+	}
+
+	save_applet(info, ret);
+}
+
 /*** StatusSpot stuff ***/
 
 static void
