@@ -6,9 +6,11 @@
 
 #include "panel-widget.h"
 #include "basep-widget.h"
-#include "snapped-widget.h"
+#include "edge-widget.h"
+#include "aligned-widget.h"
+#include "sliding-widget.h"
+#include "floating-widget.h"
 #include "drawer-widget.h"
-#include "corner-widget.h"
 
 #include "button-widget.h"
 
@@ -21,11 +23,16 @@
 #include "panel_config_global.h"
 
 #include "menu.h"
+#include "menu-fentry.h"
+#include "menu-rh.h"
+#include "menu-properties.h"
+#include "menu-util.h"
 #include "drawer.h"
 #include "swallow.h"
 #include "launcher.h"
 #include "logout.h"
 #include "extern.h"
+#include "status.h"
 
 #include "panel-util.h"
 
