@@ -11,11 +11,13 @@ typedef GNOME_Panel_OrientType PanelOrientType;
 #define ORIENT_LEFT GNOME_Panel_ORIENT_LEFT
 #define ORIENT_RIGHT GNOME_Panel_ORIENT_RIGHT
 
-typedef GNOME_Panel_SizeType PanelSizeType;
-#define SIZE_TINY GNOME_Panel_SIZE_TINY
-#define SIZE_STANDARD GNOME_Panel_SIZE_STANDARD
-#define SIZE_LARGE GNOME_Panel_SIZE_LARGE
-#define SIZE_HUGE GNOME_Panel_SIZE_HUGE
+enum {
+	SIZE_TINY = 24,
+	SIZE_SMALL = 36,
+	SIZE_STANDARD = 48,
+	SIZE_LARGE = 64,
+	SIZE_HUGE = 80
+};
 
 typedef GNOME_Panel_BackType PanelBackType;
 #define PANEL_BACK_NONE GNOME_Panel_BACK_NONE
