@@ -43,11 +43,6 @@ extern GlobalConfig global_config;
 
 extern GtkTooltips *panel_tooltips;
 
-/*???? this might be ugly, but I guess we can safely assume that we can only
-  have one menu open and that nothing weird will happen to the panel that
-  opened that menu whilethe user is looking over the choices*/
-extern PanelWidget *current_panel;
-
 extern char *kde_menudir;
 extern char *kde_icondir;
 extern char *kde_mini_icondir;
@@ -78,6 +73,9 @@ static GtkTargetEntry panel_drop_types[] = {
 static gint n_panel_drop_types = 
    sizeof(panel_drop_types) / sizeof(panel_drop_types[0]);
 
+static GtkTargetList *panel_target_list = NULL;
+
+
 static void
 change_window_cursor(GdkWindow *window, GdkCursorType cursor_type)
 {
@@ -212,7 +210,7 @@ orientation_change(AppletInfo *info, PanelWidget *panel)
 	} else if(info->type == APPLET_DRAWER) {
 		Drawer *drawer = info->data;
 		BasePWidget *basep = BASEP_WIDGET(drawer->drawer);
-		set_drawer_applet_orient(drawer,get_applet_orient(panel));
+		set_drawer_applet_orient(drawer, get_applet_orient(panel));
 		gtk_widget_queue_resize(drawer->drawer);
 		gtk_container_foreach(GTK_CONTAINER(basep->panel),
 				      orient_change_foreach,
@@ -433,13 +431,22 @@ state_hide_foreach(GtkWidget *w, gpointer data)
 	if(info->type == APPLET_DRAWER) {
 		Drawer *drawer = info->data;
 		BasePWidget *basep = BASEP_WIDGET(drawer->drawer);
+		GtkWidget *widget = GTK_WIDGET(basep);
 
 		DRAWER_POS (basep->pos)->temp_hidden = TRUE;
 		gtk_container_foreach(GTK_CONTAINER(basep->panel),
 				      state_hide_foreach,
 				      NULL);
 
-		gtk_widget_queue_resize (GTK_WIDGET (basep));
+		gtk_widget_queue_resize (widget);
+
+		/* quickly hide the window from sight, the allocation
+		   and all that will get updated in the main loop */
+		if(widget->window) {
+			gdk_window_move(widget->window,
+					-widget->allocation.width - 1,
+					-widget->allocation.height - 1);
+		}
 	}
 }
 
@@ -600,15 +607,11 @@ panel_destroy(GtkWidget *widget, gpointer data)
 		base_panels--;
 	}
 
-	if (current_panel == panel)
-		current_panel = NULL;
-
 	if(pd->menu)
 		gtk_widget_destroy(pd->menu);
 	
 	panel_list = g_slist_remove(panel_list,pd);
 	g_free(pd);
-	
 }
 
 static void
@@ -629,34 +632,32 @@ panel_applet_draw(GtkWidget *panel, GtkWidget *widget, gpointer data)
 }
 
 static GtkWidget *
-panel_menu_get(PanelData *pd)
+panel_menu_get(PanelWidget *panel, PanelData *pd)
 {
 	if(pd->menu)
 		return pd->menu;
 	
-	pd->menu = create_panel_root_menu(pd->panel,TRUE);
+	pd->menu = create_panel_root_menu(panel, TRUE);
 	gtk_signal_connect(GTK_OBJECT(pd->menu), "deactivate",
 			   GTK_SIGNAL_FUNC(menu_deactivate),pd);
 	return pd->menu;
 }
 
 GtkWidget *
-make_popup_panel_menu (void)
+make_popup_panel_menu (PanelWidget *panel)
 {
 	BasePWidget *basep;
 	PanelData *pd;
 	GtkWidget *menu;
 
-	if (!current_panel) {
+	if (!panel) {
 	        basep = BASEP_WIDGET (((PanelData *)panel_list->data)->panel);			
-		current_panel = PANEL_WIDGET (basep->panel);
+		panel = PANEL_WIDGET (basep->panel);
 	} else
-		basep = BASEP_WIDGET (current_panel->panel_parent);
+		basep = BASEP_WIDGET (panel->panel_parent);
 	
 	pd = gtk_object_get_user_data (GTK_OBJECT (basep));
-	menu = panel_menu_get (pd);
-	gtk_object_set_data (GTK_OBJECT (menu), "menu_panel",
-			     basep->panel);
+	menu = panel_menu_get (panel, pd);
 	pd->menu_age = 0;
 	return menu;
 }
@@ -673,9 +674,7 @@ panel_event(GtkWidget *widget, GdkEvent *event, PanelData *pd)
 		case 3: /* fall through */
 			if(!panel_applet_in_drag) {
 				GtkWidget *menu;
-				current_panel =
-					PANEL_WIDGET(basep->panel);
-				menu = make_popup_panel_menu ();
+				menu = make_popup_panel_menu (PANEL_WIDGET(basep->panel));
 				BASEP_WIDGET(basep)->autohide_inhibit 
 					= TRUE;
 				basep_widget_autohide (
@@ -910,8 +909,12 @@ drop_internal_applet(PanelWidget *panel, int pos, char *applet_type)
 {
 	if(!applet_type)
 		return;
-	if(strcmp(applet_type,"MENU:MAIN")==0) {
-		drop_menu(panel, pos, NULL);
+	if(strncmp(applet_type,"MENU:",strlen("MENU:"))==0) {
+		char *menu = &applet_type[strlen("MENU:")];
+		if(strcmp(menu,"MAIN")==0)
+			drop_menu(panel, pos, NULL);
+		else
+			drop_menu(panel, pos, menu);
 	} else if(strcmp(applet_type,"DRAWER:NEW")==0) {
 		load_drawer_applet(-1, NULL, NULL, panel, pos, TRUE);
 	} else if(strcmp(applet_type,"LOGOUT:NEW")==0) {
@@ -942,14 +945,127 @@ drop_color(PanelWidget *panel, int pos, guint16 *dropped)
 	panel_widget_set_back_color(panel, &c);
 }
 
+static gboolean
+is_this_drop_ok(GtkWidget *widget, GdkDragContext *context, guint *info,
+		GdkAtom *ret_atom)
+{
+	GtkWidget *source_widget;
+	GtkWidget *panel; /*PanelWidget*/
+	GList *li;
+
+	g_return_val_if_fail(widget!=NULL, FALSE);
+	g_return_val_if_fail(IS_BASEP_WIDGET(widget), FALSE);
+
+	if(!(context->actions & GDK_ACTION_COPY))
+		return FALSE;
+
+	panel = BASEP_WIDGET (widget)->panel;
+
+	source_widget = gtk_drag_get_source_widget (context);
+	/* if this is a drag from this panel to this panel of a launcher,
+	   then ignore it */
+	if(source_widget && IS_BUTTON_WIDGET(source_widget) &&
+	   source_widget->parent == panel) {
+		return FALSE;
+	}
+
+	if(!panel_target_list)
+		panel_target_list = gtk_target_list_new(panel_drop_types,
+							n_panel_drop_types);
+	for(li = context->targets; li; li = li->next) {
+		guint temp_info;
+		if(gtk_target_list_find(panel_target_list, 
+					GPOINTER_TO_INT(li->data),
+					&temp_info)) {
+			if(info) *info = temp_info;
+			if(ret_atom) *ret_atom = GPOINTER_TO_INT(li->data);
+			break;
+		}
+	}
+	/* if we haven't found it */
+	if(!li)
+		return FALSE;
+	return TRUE;
+}
+
 static void
-panel_widget_dnd_drop_internal (GtkWidget	 *widget,
-				GdkDragContext   *context,
-				gint              x,
-				gint              y,
-				GtkSelectionData *selection_data,
-				guint             info,
-				guint             time)
+do_highlight (GtkWidget *widget, gboolean highlight)
+{
+	gboolean have_drag;
+	have_drag = GPOINTER_TO_INT(gtk_object_get_data (GTK_OBJECT (widget),
+							 "have-drag"));
+	if(highlight) {
+		if(!have_drag) {
+			gtk_object_set_data (GTK_OBJECT (widget), "have-drag",
+					     GINT_TO_POINTER (TRUE));
+			gtk_drag_highlight (widget);
+		}
+	} else {
+		if(have_drag) {
+			gtk_object_remove_data (GTK_OBJECT (widget),
+						"have-drag");
+			gtk_drag_unhighlight (widget);
+		}
+	}
+}
+
+
+static gboolean
+drag_motion_cb(GtkWidget	  *widget,
+	       GdkDragContext     *context,
+	       gint                x,
+	       gint                y,
+	       guint               time)
+{
+	gdk_drag_status (context, GDK_ACTION_COPY, time);
+
+	if(!is_this_drop_ok(widget, context, NULL, NULL))
+		return FALSE;
+
+	do_highlight(widget, TRUE);
+
+	basep_widget_autoshow(BASEP_WIDGET(widget));
+	basep_widget_queue_autohide(BASEP_WIDGET(widget));
+
+	return TRUE;
+}
+
+static gboolean
+drag_drop_cb(GtkWidget	       *widget,
+	     GdkDragContext    *context,
+	     gint               x,
+	     gint               y,
+	     guint              time,
+	     Launcher *launcher)
+{
+	GdkAtom ret_atom = 0;
+
+	if(!is_this_drop_ok(widget, context, NULL, &ret_atom))
+		return FALSE;
+
+	gtk_drag_get_data(widget, context,
+			  ret_atom, time);
+
+	return TRUE;
+}
+
+static void  
+drag_leave_cb(GtkWidget	       *widget,
+	      GdkDragContext   *context,
+	      guint             time,
+	      Launcher *launcher)
+{
+	do_highlight (widget, FALSE);
+}
+
+static void
+drag_data_recieved_cb (GtkWidget	 *widget,
+		       GdkDragContext   *context,
+		       gint              x,
+		       gint              y,
+		       GtkSelectionData *selection_data,
+		       guint             info,
+		       guint             time)
 {
 	PanelWidget *panel;
 	int pos;
@@ -957,6 +1073,14 @@ panel_widget_dnd_drop_internal (GtkWidget	 *widget,
 	g_return_if_fail(widget!=NULL);
 	g_return_if_fail(IS_BASEP_WIDGET(widget));
 
+	/* we use this only to really find out the info, we already
+	   know this is an ok drop site and the info that got passed
+	   to us is bogus (it's always 0 in fact) */
+	if(!is_this_drop_ok(widget, context, &info, NULL)) {
+		gtk_drag_finish(context,FALSE,FALSE,time);
+		return;
+	}
+
 	panel = PANEL_WIDGET (BASEP_WIDGET (widget)->panel);
 
 	pos = panel_widget_get_cursorloc(panel);
@@ -983,8 +1107,10 @@ panel_widget_dnd_drop_internal (GtkWidget	 *widget,
 		drop_menu(panel, pos, (char *)selection_data->data);
 		break;
 	case TARGET_APPLET:
-		if(!selection_data->data)
+		if(!selection_data->data) {
+			gtk_drag_finish(context,FALSE,FALSE,time);
 			return;
+		}
 		load_extern_applet((char *)selection_data->data, NULL,
 				   panel, pos, TRUE, FALSE);
 		break;
@@ -992,6 +1118,8 @@ panel_widget_dnd_drop_internal (GtkWidget	 *widget,
 		drop_internal_applet(panel, pos, (char *)selection_data->data);
 		break;
 	}
+
+	gtk_drag_finish(context,TRUE,FALSE,time);
 }
 
 static void
@@ -1070,6 +1198,27 @@ floating_size_alloc(BasePWidget *basep, GtkAllocation *alloc, gpointer data)
 			      basep->panel);
 }
 
+static void
+drawer_orient_change_foreach(GtkWidget *w, gpointer data)
+{
+	AppletInfo *info = gtk_object_get_data(GTK_OBJECT(w), "applet_info");
+	PanelWidget *panel = data;
+	
+	if(info->type == APPLET_DRAWER)
+		orientation_change(info,panel);
+}
+
+static void
+sliding_size_alloc(BasePWidget *basep, GtkAllocation *alloc, gpointer data)
+{
+	if(!GTK_WIDGET_REALIZED(basep))
+		return;
+
+	gtk_container_foreach(GTK_CONTAINER(basep->panel),
+			      drawer_orient_change_foreach,
+			      basep->panel);
+}
+
 void
 panel_setup(GtkWidget *panelw)
 {
@@ -1125,15 +1274,29 @@ panel_setup(GtkWidget *panelw)
 
 	gtk_signal_connect(GTK_OBJECT(basep),
 			   "drag_data_received",
-			   GTK_SIGNAL_FUNC(panel_widget_dnd_drop_internal),
+			   GTK_SIGNAL_FUNC(drag_data_recieved_cb),
+			   NULL);
+	gtk_signal_connect(GTK_OBJECT(basep),
+			   "drag_motion",
+			   GTK_SIGNAL_FUNC(drag_motion_cb),
+			   NULL);
+	gtk_signal_connect(GTK_OBJECT(basep),
+			   "drag_leave",
+			   GTK_SIGNAL_FUNC(drag_leave_cb),
+			   NULL);
+	gtk_signal_connect(GTK_OBJECT(basep),
+			   "drag_drop",
+			   GTK_SIGNAL_FUNC(drag_drop_cb),
 			   NULL);
 
-	gtk_drag_dest_set (GTK_WIDGET (basep),
+	/*gtk_drag_dest_set (GTK_WIDGET (basep),
 			   GTK_DEST_DEFAULT_MOTION |
 			   GTK_DEST_DEFAULT_HIGHLIGHT |
 			   GTK_DEST_DEFAULT_DROP,
 			   panel_drop_types, n_panel_drop_types,
-			   GDK_ACTION_COPY);
+			   GDK_ACTION_COPY);*/
+	gtk_drag_dest_set (GTK_WIDGET (basep),
+			   0, NULL, 0, 0);
 
 	gtk_signal_connect (GTK_OBJECT (basep),
 			    "state_change",
@@ -1165,10 +1328,16 @@ panel_setup(GtkWidget *panelw)
 					 GTK_SIGNAL_FUNC(panel_realize),
 					 NULL);
 
-	if(IS_FLOATING_WIDGET(panelw))
+	if(IS_FLOATING_WIDGET(panelw) ||
+	   IS_DRAWER_WIDGET(panelw))
 		gtk_signal_connect_after(GTK_OBJECT(panelw), "size_allocate",
 					 GTK_SIGNAL_FUNC(floating_size_alloc),
 					 NULL);
+	if(IS_SLIDING_WIDGET(panelw) ||
+	   IS_ALIGNED_WIDGET(panelw))
+		gtk_signal_connect_after(GTK_OBJECT(panelw), "size_allocate",
+					 GTK_SIGNAL_FUNC(sliding_size_alloc),
+					 NULL);
 }
 
 /*send state change to all the panels*/
