@@ -12,7 +12,7 @@ struct _GlobalConfig {
 	int minimized_size;
 	int minimize_delay;
 	int tooltips_enabled;
-	int show_small_icons;
+	/*int show_small_icons;*/
 	int show_dot_buttons;
 	int hungry_menus;
 	int off_panel_popups;
