@@ -229,23 +229,42 @@ send_applet_session_save (AppletInfo *info,
 			  const char *cfgpath,
 			  const char *globcfgpath)
 {
+	guint timeout;
 	CORBA_Environment ev;
 	
 	/*new unique cookie*/
 	ss_cookie++;
 	
 #ifdef PANEL_DEBUG	
-	printf("SENDING_SESSION_SAVE (%u)\n",ss_cookie);
+	printf("SENDING_SESSION_SAVE (%u)\n", ss_cookie);
 #endif
 
-	gtk_timeout_add(ss_timeout,session_save_timeout,GINT_TO_POINTER((int)ss_cookie));
+	timeout = gtk_timeout_add(ss_timeout, session_save_timeout,
+				  GINT_TO_POINTER((int)ss_cookie));
 
 	CORBA_exception_init(&ev);
 	GNOME_Applet_save_session(obj,
 				  (CORBA_char *)cfgpath,
 				  (CORBA_char *)globcfgpath,
 				  ss_cookie, &ev);
-	if(ev._major) {
+	if(ev._major == CORBA_SYSTEM_EXCEPTION) {
+		CORBA_SystemException *exc =
+			CORBA_exception_value(&ev);
+		if(exc &&
+		   (exc->minor == ex_CORBA_BAD_OPERATION ||
+		    exc->minor == ex_CORBA_NO_IMPLEMENT)) {
+			gboolean ret;
+			gtk_timeout_remove(timeout);
+			CORBA_exception_free(&ev);
+			CORBA_exception_init(&ev);
+			ret = GNOME_Applet_session_save(obj,
+						(CORBA_char *)cfgpath,
+						(CORBA_char *)globcfgpath,
+						&ev);
+			save_applet(info, ret);
+		}
+	} else if(ev._major) {
+		gtk_timeout_remove(timeout);
 		panel_clean_applet(info);
 		CORBA_exception_free(&ev);
 		save_next_applet();
@@ -272,11 +291,13 @@ save_applet_configuration(AppletInfo *info)
 	buf = g_string_new(NULL);
 
 	gnome_config_push_prefix("");
-	g_string_sprintf(buf, "%sApplet_Config/Applet_%d", PANEL_CONFIG_PATH, info->applet_id+1);
+	g_string_sprintf(buf, "%sApplet_Config/Applet_%d", PANEL_CONFIG_PATH,
+			 info->applet_id+1);
 	gnome_config_clean_section(buf->str);
 	gnome_config_pop_prefix();
 
-	g_string_sprintf(buf, "%sApplet_Config/Applet_%d/", PANEL_CONFIG_PATH, info->applet_id+1);
+	g_string_sprintf(buf, "%sApplet_Config/Applet_%d/", PANEL_CONFIG_PATH,
+			 info->applet_id+1);
 	gnome_config_push_prefix(buf->str);
 
 	/*obviously no need for saving*/
@@ -302,7 +323,6 @@ save_applet_configuration(AppletInfo *info)
 	switch(info->type) {
 	case APPLET_EXTERN:
 		{
-			char *globalcfg;
 			char *s;
 			Extern *ext = info->data;
 
@@ -310,9 +330,6 @@ save_applet_configuration(AppletInfo *info)
 			gnome_config_set_string("id", EMPTY_ID);
 			gnome_config_pop_prefix();
 
-			globalcfg = g_concat_dir_and_file(PANEL_CONFIG_PATH,
-							  "Applet_All_Extern/");
-
 			/*this is the file path we must kill first */
 			g_string_sprintf(buf, "%sApplet_%d_Extern",
 					 PANEL_CONFIG_PATH, info->applet_id+1);
@@ -335,8 +352,9 @@ save_applet_configuration(AppletInfo *info)
 					 PANEL_CONFIG_PATH, info->applet_id+1);
 			/*have the applet do it's own session saving*/
 			send_applet_session_save(info,ext->applet,
-						 buf->str, globalcfg);
-			g_free(globalcfg);
+						 buf->str,
+						 PANEL_CONFIG_PATH
+						 "Applet_All_Extern/");
 			return FALSE; /*here we'll wait for done_session_save*/
 		}
 	case APPLET_DRAWER: 
@@ -536,7 +554,6 @@ do_session_save(GnomeClient *client,
 		int sync_panels)
 {
 	int num;
-	char *s;
 #if PER_SESSION_CONFIGURATION
 	gchar *new_args[] = { "rm", "-r", NULL };
 #endif /* PER_SESSION_CONFIGURATION */
@@ -565,9 +582,7 @@ do_session_save(GnomeClient *client,
 	printf(" 2"); fflush(stdout);
 #endif
 
-	s = g_concat_dir_and_file(PANEL_CONFIG_PATH,"panel/Config/");
-	gnome_config_push_prefix (s);
-	g_free(s);
+	gnome_config_push_prefix (PANEL_CONFIG_PATH "panel/Config/");
 
 	if(complete_sync || sync_applets)
 		gnome_config_set_int ("applet_count", applet_count);
@@ -721,70 +736,74 @@ load_default_applets1(PanelWidget *panel)
 		NULL };
 	int i;
 	char *p;
+	int sz;
+
+	if(gdk_screen_width()<800)
+		sz = SIZE_TINY;
+	else if(gdk_screen_width()<1024)
+		sz = SIZE_SMALL;
+	else
+		sz = SIZE_STANDARD;
 
 	load_menu_applet(NULL, get_default_menu_flags (), 
-			 panels->data, 0, FALSE);
+			 panels->data, 0, TRUE);
+
+	/*load up some buttons, but only if screen larger then 639*/
+	if(gdk_screen_width()>=640) {
+		load_logout_applet(panel, sz, TRUE);
+		load_lock_applet(panel, sz*2, TRUE);
 
-	/* if bigger then 640+50+50 add a logout button */
-	if(gdk_screen_width()>640+50)
-		load_logout_applet(panel, 48, FALSE);
-	/* if bigger then 640+100+50 add a lock button */
-	if(gdk_screen_width()>640+100+50)
-		load_lock_applet(panel, 48*2, FALSE);
-	
-	if(gdk_screen_width()>639) {
-		/*load up some launchers, but only if screen larger then 639*/
 		for(i=0;def_launchers[i]!=NULL;i++) {
 			p = gnome_datadir_file (def_launchers[i]);
 			/*int center = gdk_screen_width()/2;*/
 			if(p) {
 				load_launcher_applet(p, panel,
-						     48*4+i*48, FALSE);
+						     sz*4+i*sz, TRUE);
 				g_free(p);
 			}
 		}
 	}
-
 	load_extern_applet("tasklist_applet", NULL,
-			   panel, INT_MAX/2/*flush right*/, FALSE, TRUE);
-	/* if we are larger then 640, put the status dock on the main
-	   panel, otherwise on the auxiliaray below */
-	if(gdk_screen_width()>640)
-		load_status_applet(panel,
-				   INT_MAX/2 + 1000/*flush right*/, FALSE);
+			   panel, INT_MAX/2/*flush right*/, TRUE, TRUE);
+	load_status_applet(panel,
+			   INT_MAX/2 + 1000/*flush right*/, TRUE);
+	/* for small panel stick the on the main panel */
+	if(gdk_screen_width()<1024)
+		load_extern_applet("gen_util_clock", NULL,
+				   panel, INT_MAX/2 + 2000, TRUE, TRUE);
 }
 
 static void
 load_default_applets2(PanelWidget *panel)
 {
 	load_extern_applet("deskguide_applet", NULL,
-			   panel, 0, FALSE, TRUE);
-	load_extern_applet("gen_util_clock", NULL,
-			   panel, 1000, FALSE, TRUE);
+			   panel, 0, TRUE, TRUE);
+	/* for small panel stick the on the main panel not this one */
+	if(gdk_screen_width()>=1024)
+		load_extern_applet("gen_util_clock", NULL,
+				   panel, 1000, TRUE, TRUE);
 	load_extern_applet("gen_util_mailcheck",NULL,
-			   panel, 2000, FALSE, TRUE);
-	/* if we are smaller or equal to 640, put the status dock on the
-	   auxiliary and not the main panel */
-	if(gdk_screen_width()<=640)
-		load_status_applet(panel, 3000, FALSE);
+			   panel, 2000, TRUE, TRUE);
+	if(g_file_exists("/proc/apm"))
+		load_extern_applet("battery_applet",NULL,
+				   panel, 4000, TRUE, TRUE);
 }
 
 void
 init_user_applets(void)
 {
 	GString *buf;
-	int   count,num;	
+	int count, num;	
 
+	count = gnome_config_get_int(PANEL_CONFIG_PATH
+				     "panel/Config/applet_count=0");
 	buf = g_string_new(NULL);
-	g_string_sprintf(buf,"%spanel/Config/applet_count=0",
-			 PANEL_CONFIG_PATH);
-	count=gnome_config_get_int(buf->str);
 	for(num=1;num<=count;num++) {
 		char *applet_name;
 		int   pos=0,panel_num;
 		PanelWidget *panel;
 
-		g_string_sprintf(buf,"%sApplet_Config/Applet_%d/",
+		g_string_sprintf(buf, "%sApplet_Config/Applet_%d/",
 				 PANEL_CONFIG_PATH, num);
 		gnome_config_push_prefix(buf->str);
 		applet_name = gnome_config_get_string("id=Unknown");
@@ -825,30 +844,30 @@ init_user_applets(void)
 				g_string_sprintf(buf,"%sApplet_%d_Extern/",
 						 PANEL_CONFIG_PATH,num);
 				load_extern_applet(goad_id, buf->str, 
-						   panel, pos, FALSE, TRUE);
+						   panel, pos, TRUE, TRUE);
 			}
 			g_free(goad_id);
 		} else if(strcmp(applet_name, LAUNCHER_ID) == 0) { 
 			char *params = gnome_config_get_string("parameters=");
-			load_launcher_applet(params, panel, pos, FALSE);
+			load_launcher_applet(params, panel, pos, TRUE);
 			g_free(params);
 		} else if(strcmp(applet_name, LOGOUT_ID) == 0) { 
-			load_logout_applet(panel, pos, FALSE);
+			load_logout_applet(panel, pos, TRUE);
 		} else if(strcmp(applet_name, LOCK_ID) == 0) {
-			load_lock_applet(panel, pos, FALSE);
+			load_lock_applet(panel, pos, TRUE);
 		} else if(strcmp(applet_name, STATUS_ID) == 0) {
-			load_status_applet(panel, pos, FALSE);
+			load_status_applet(panel, pos, TRUE);
 		} else if(strcmp(applet_name, SWALLOW_ID) == 0) {
 			char *path = gnome_config_get_string("execpath=");
 			char *params = gnome_config_get_string("parameters=");
 			int width = gnome_config_get_int("width=0");
 			int height = gnome_config_get_int("height=0");
 			load_swallow_applet(path, params, width, height,
-					    panel, pos, FALSE);
+					    panel, pos, TRUE);
 			g_free(path);
 			g_free(params);
 		} else if(strcmp(applet_name, MENU_ID) == 0) {
-			char *params = gnome_config_get_string("parameters=");
+			char *params = gnome_config_get_string("parameters=.");
 			char *s;
 			int type =
 				gnome_config_get_int("main_menu_type=-1");
@@ -909,14 +928,14 @@ init_user_applets(void)
 					MAIN_MENU_DESKTOP_SUB;
 			}
 
-			load_menu_applet(params, flags, panel, pos, FALSE);
+			load_menu_applet(params, flags, panel, pos, TRUE);
 			g_free(params);
 		} else if(strcmp(applet_name, DRAWER_ID) == 0) {
 			int mypanel = gnome_config_get_int("parameters=-1");
 			char *pixmap = gnome_config_get_string("pixmap=");
 			char *tooltip = gnome_config_get_string("tooltip=");
 			load_drawer_applet(mypanel, pixmap, tooltip,
-					   panel, pos, FALSE);
+					   panel, pos, TRUE);
 			g_free(pixmap);
 			g_free(tooltip);
 		} else
@@ -931,25 +950,37 @@ void
 init_user_panels(void)
 {
 	GString *buf;
-	int   count,num;	
+	int count, num;	
 	GtkWidget *panel=NULL;
 
-	buf = g_string_new(NULL);
-	g_string_sprintf(buf,"%spanel/Config/panel_count=0",
-			 PANEL_CONFIG_PATH);
-	count=gnome_config_get_int(buf->str);
+	count = gnome_config_get_int(PANEL_CONFIG_PATH
+				     "panel/Config/panel_count=0");
 
 	/*load a default snapped panel on the bottom of the screen,
 	  it is required to have at least one panel for this all
 	  to work, so this is the way we find out if there was no
 	  config from last time*/
 	if(count<=0)  {
+		int sz;
+		gboolean hidebutton_pixmaps;
+
+		if(gdk_screen_width()<800) {
+			sz = SIZE_TINY;
+			hidebutton_pixmaps = FALSE;
+		} else if(gdk_screen_width()<1024) {
+			sz = SIZE_SMALL;
+			hidebutton_pixmaps = TRUE;
+		} else {
+			sz = SIZE_STANDARD;
+			hidebutton_pixmaps = TRUE;
+		}
+
 		panel = edge_widget_new(BORDER_BOTTOM,
 					BASEP_EXPLICIT_HIDE,
 					BASEP_SHOWN,
-					SIZE_STANDARD,
-					TRUE,
+					sz,
 					TRUE,
+					hidebutton_pixmaps,
 					PANEL_BACK_NONE,
 					NULL,
 					TRUE,
@@ -962,13 +993,24 @@ init_user_panels(void)
 		/*load up default applets on the default panel*/
 		load_default_applets1(PANEL_WIDGET(BASEP_WIDGET(panel)->panel));
 
+		if(gdk_screen_height()<600) {
+			sz = SIZE_TINY;
+			hidebutton_pixmaps = FALSE;
+		} else if(gdk_screen_height()<768) {
+			sz = SIZE_SMALL;
+			hidebutton_pixmaps = TRUE;
+		} else {
+			sz = SIZE_STANDARD;
+			hidebutton_pixmaps = TRUE;
+		}
+
 		panel = aligned_widget_new(ALIGNED_LEFT,
 					   BORDER_RIGHT,
 					   BASEP_EXPLICIT_HIDE,
 					   BASEP_SHOWN,
-					   SIZE_STANDARD,
-					   TRUE,
+					   sz,
 					   TRUE,
+					   hidebutton_pixmaps,
 					   PANEL_BACK_NONE,
 					   NULL,
 					   TRUE,
@@ -986,10 +1028,11 @@ init_user_panels(void)
 		  next syncing*/
 		need_complete_save = TRUE;
 
-		g_string_free(buf,TRUE);
 		return;
 	}
 
+	buf = g_string_new(NULL);
+
 	for(num=1;num<=count;num++) {
 		PanelType type;
 		PanelBackType back_type;
@@ -1453,17 +1496,16 @@ convert_read_old_config(void)
 	int applet_count; /*store this so that we can clean*/
 	int panel_count; /*store this so that we can clean*/
 
-	buf = g_string_new(NULL);
-	g_string_sprintf(buf,"%spanel/Config/",PANEL_CONFIG_PATH);
-	gnome_config_push_prefix(buf->str);
+	gnome_config_push_prefix(PANEL_CONFIG_PATH "panel/Config/");
 
 	gnome_config_get_bool_with_default("tooltips_enabled=TRUE",&is_def);
 	if(is_def) {
 		gnome_config_pop_prefix();
-		g_string_free(buf,TRUE);
 		return FALSE;
 	}
 	
+	buf = g_string_new(NULL);
+
 	global_config.tooltips_enabled =
 		gnome_config_get_bool("tooltips_enabled=TRUE");
 
@@ -1539,14 +1581,12 @@ convert_read_old_config(void)
 
 	gnome_config_push_prefix("");
 
-	g_string_sprintf(buf,"%spanel/Config",PANEL_CONFIG_PATH);
+	gnome_config_clean_section(PANEL_CONFIG_PATH "panel/Config");
 
-	gnome_config_clean_section(buf->str);
-
-	g_string_sprintf(buf,"%spanel/Config/applet_count",PANEL_CONFIG_PATH);
-	gnome_config_set_int(buf->str,applet_count);
-	g_string_sprintf(buf,"%spanel/Config/panel_count",PANEL_CONFIG_PATH);
-	gnome_config_set_int(buf->str,panel_count);
+	gnome_config_set_int(PANEL_CONFIG_PATH "panel/Config/applet_count",
+			     applet_count);
+	gnome_config_set_int(PANEL_CONFIG_PATH "panel/Config/panel_count",
+			     panel_count);
 
 	gnome_config_pop_prefix();
 
